/*
 *	arg.c - argument processing support functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: arg.c,v 1.4 92/09/09 10:52:24 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


/*
 * entpid() - enter Process IDentifier for searching
 */

int
entpid(p)
	char *p;			/* PID string pointer */
{
	int i, pid;
	char *s;

	for (s = p; *s;) {
	/*
	 * Assemble Process IDentifier.
	 */
		for (pid = 0; *s && *s != ','; *s++) {
			if ( ! isascii(*s) || ! isdigit(*s)) {
				(void) fprintf(stderr,
					"%s: illegal PID specification: %s\n",
					Pn, p);
				return(1);
			}
			pid = (pid * 10) + *s - '0';
		}
		if (*s)
			s++;
	/*
	 * Avoid entering duplicates.
	 */
		for (i = 0; i < Npid; i++) {
			if (pid == Spid[i])
				break;
		}
		if (i < Npid)
			continue;
	/*
	 * Allocate PID table space.
	 */
		if (Npid >= Mxpid) {
			Mxpid += PIDINCR;
			if (Spid == NULL)
				Spid = (int *)malloc((MALLOC_S)
					(sizeof(int *) * Mxpid));
			else
				Spid = (int *)realloc((MALLOC_P *)Spid,
					(MALLOC_S)(sizeof(int *) * Mxpid));
			if (Spid == NULL) {
				(void) fprintf(stderr, "%s: no space for PIDs",
					Pn);
				exit(1);
			}
		}
		Spid[Npid++] = pid;
	}
	return(0);
}


/*
 * entuid() - enter User Identifier for searching
 */

int
entuid(u)
	char *u;			/* User IDentifier string pointer */
{
	int err, i, n;
	struct passwd *pw;
	char *s;
	char uid[LOGINML];

	for (err = 0, s = u; *s;) {
	/*
	 * Assemble next User IDentifier.
	 */
		for (i = n = 0; *s && *s != ','; i++, s++) {
			if (i >= LOGINML-1) {
				(void) fprintf(stderr,
					"%s: illegal UID in %s\n", Pn, u);
				return(1);
			}
			uid[i] = *s;
			if (n < 0)
				continue;
			if (isascii(*s) && isdigit(*s))
				n = (n * 10) + *s - '0';
			else
				n = -1;
		}
		if (*s)
			s++;
		if (n < 0) {
		       uid[i] = '\0';
		       if ((pw = getpwnam(uid)) == NULL) {
				(void) fprintf(stderr,
					"%s: can't get UID for %s\n",
					Pn, uid);
				err = 1;
				continue;
			} else
				n = pw->pw_uid;
		}

#if	defined(HASSECURITY)
/*
 * If the security mode is enabled, only the root user may list files
 * belonging to user IDs other than the real user ID of this lsof process.
 */
	if (Myuid && n != Myuid) {
		(void) fprintf(stderr,
		    "%s: ID %d request rejected because of security mode.\n",
		    Pn, n);
		err = 1;
		continue;
	}
#endif	/* HASSECURITY */

	/*
	 * Avoid entering duplicates.
	 */
		for (i = 0; i < Nuid; i++) {
			if (n == Suid[i])
				break;
		}
		if (i < Nuid)
			continue;
	/*
	 * Allocate space for User IDentifier.
	 */
		if (Nuid >= Mxuid) {
			Mxuid += UIDINCR;
			if (Suid == NULL)
				Suid = (int *)malloc((MALLOC_S)
					(sizeof(int) * Mxuid));
			else
				Suid = (int *)realloc((MALLOC_P *)Suid,
					(MALLOC_S)(sizeof(int) * Mxuid));
			if (Suid == NULL) {
				(void) fprintf(stderr, "%s: no space for UIDs",
					Pn);
				exit(1);
			}
		}
		Suid[Nuid++] = n;
	}
	return(err);
}


/*
 * usage() - display usage and exit
 */

void
usage(xv)
	int xv;				/* exit value */
{
	(void) fprintf(stderr, "%s %s usage: [-dhl%snNost]", Pn, Version,

#if	defined(DEHNSSG)
		"");
#endif

#if	defined(_IBMR2)
		"L");
#endif

	(void) fprintf(stderr, " [-c c]");

#if	defined(HASKOPT)
	(void) fprintf(stderr, " [-k k]");
#endif

	(void) fprintf(stderr, " [-p l] [-u l] [-U] [names]\n");
	(void) fprintf(stderr,
		"\t-c c   read kernel memory from c (default = %s)\n",

#if	defined(DEHINSG)
		"/dev/kmem"
#endif

#if	defined(sun)
		"/dev/mem"
#endif

		);
	(void) fprintf(stderr,
		"\t-d     issue /dev subdirectory error messages\n");
	(void) fprintf(stderr,
		"\t-h     list help\n");

#if	defined(HASKOPT)
	(void) fprintf(stderr,
		"\t-k k   read kernel symbols from k (default = %s)\n",

# if	defined(NeXT)
		VMUNIX
# endif

# if	defined(DYNIX) || defined(hpux) || defined(sun)
		N_UNIX
# endif

		);
#endif

	(void) fprintf(stderr,
		"\t-l     do not convert UID's to login names\n");

#if	defined(_IBMR2)
	(void) fprintf(stderr,
		"\t-L     display loader references to the same file\n");
#endif

	(void) fprintf(stderr,
		"\t-n     list Internet network files\n");
	(void) fprintf(stderr,
		"\t-N     list NFS files\n");
	(void) fprintf(stderr,
		"\t-p l   list files for comma-separated Process ID list l\n");
	(void) fprintf(stderr,
		"\t-o     print current file offset instead of file size\n");
	(void) fprintf(stderr,
 		"\t-s     do not display file size or offset\n");
	(void) fprintf(stderr,
		"\t-t     terse output -- Process ID numbers and no header\n");
	(void) fprintf(stderr,
		"\t-u l   list files for comma-separated user list l ");
	(void) fprintf(stderr,
		"(names or UIDs)\n");
	(void) fprintf(stderr,
		"\t-U     list Unix domain sockets\n");
	(void) fprintf(stderr,
		"\tnames  path names of file systems, mount points, devices");
	(void) fprintf(stderr,
		" and\n\t       files to locate\n");
	(void) fprintf(stderr,
		"The security mode is %s.\n",

#if	defined(HASSECURITY)
		"enabled -- only the root user may list all open files"
#else	/* ! HASSECURITY */
		"disabled -- anyone may list all open files"
#endif	/* HASSECURITY */

		);
	exit(xv);
}
