/*
 *	dev.c - device support functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: dev.c,v 1.3 92/09/06 11:14:21 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"

static int compdev();
static void stkdir();


/*
 * compdev() - compare device numbers
 */

static int
compdev(p1, p2)
	struct dev **p1;
	struct dev **p2;
{
	if ((long)((*p1)->rdev) < (long)((*p2)->rdev))
		return(-1);
	if ((long)((*p1)->rdev) > (long)((*p2)->rdev))
		return(1);
	return(0);
}


/*
 * printchdevname() - print character device name
 */

int
printchdevname(rdev, chan)
	dev_t rdev;			/* device */

#if	defined(DEHNSSG)
	int chan;			/* dummy */
#endif

#if	defined(_IBMR2)
	chan_t chan;			/* channel */
#endif

/* ARGSUSED */

{

#if	defined(sgi) || defined(sun)
	struct clone *c;
#endif

	int low, hi, mid;
	struct dev *dp;

#if	defined(sgi)
/*
 * Search for SGI clone.
 */
	if (Ntype == N_COM && Clone) {
		for (c = Clone; c; c = c->next) {
			dp = c->dp;
			if (major(dp->rdev) == CLN2_MAJOR
			&&  major(rdev) == minor(dp->rdev))
				break;
			if (major(rdev) == major(dp->rdev))
				break;
		}
		if (c) {
			(void) strcpy(Namech, dp->name);
			return(0);
		}
	}
#endif	/* sgi */


#if	defined(sun)
/*
 * Search for SunOS clone parent.
 */
	if (Ntype == N_STREAM && Clone) {
		for (c = Clone; c; c = c->next) {
			dp = c->dp;
			if (major(rdev) == minor(dp->rdev)) {
				(void) strcpy(Namech, dp->name);
				return(0);
			}
		}
	}
#endif	/* sun */

	low = mid = 0;
	hi = Ndev - 1;
        while (low <= hi) {
                mid = (low + hi) / 2;
		dp = Sdev[mid];
		if ((long)rdev < (long)dp->rdev)
			hi = mid - 1;
		else if ((long)rdev > (long)dp->rdev)
                        low = mid + 1;
                else {

#if	defined(DEHNSSG)
			(void) strcpy(Namech, dp->name);
#endif

#if	defined(_IBMR2)
			if (chan < 0)
				(void) strcpy(Namech, dp->name);
			else
				(void) sprintf(Namech, "%s/%d", dp->name,
					chan);
#endif

			return(0);
		}
        }
	(void) strcpy(Namech, "name unknown");
	return(1);
}


#if	defined(DTYPE_GNODE)
/*
 * printdevice() - print device file
 */

void
printdevice(va)
	caddr_t va;			/* vnode's kernel address */
{
	struct vnode v;

	if (readvnode(va, &v)) {
                printline();
                return;
        }
	(void) strcpy(Type, "dev");
	printline();
}
#endif


/*
 * readdev() - read names, modes and device types of everything in /dev
 */

void
readdev()
{

	DIR *dfp;
	int dn = 0;
	char **dstk = NULL;
	int dx = 0;
	int err = 0;
	int i = 0;
	MALLOC_S nl;
	char path[MAXNAMLEN+1];
	int pl;
	struct stat sb;

#if	defined(DYNIX) || defined(NeXT) || defined(sgi)
	struct direct *dp;
#endif

#if	defined(eta10)
	struct bsddirect *dp;
#endif

#if	defined(hpux) || defined(_IBMR2) || defined(sun)
	struct dirent *dp;
#endif

#if	defined(sgi) || defined(sun)
	struct clone *c;
#endif

	(void) stkdir(&dstk, &dn, &dx, "/dev");
/*
 * Unstack the next /dev or /dev/<subdirectory> directory.
 */
	while (--dx >= 0) {
		(void) strcpy(path, dstk[dx]);
		if ((dfp = opendir(path)) == NULL) {
			if (Fdevmsg)
				(void) fprintf(stderr, "%s: can't open %s\n",
					Pn, path);
			continue;
		}
		(void) strcat(path, "/");
		pl = strlen(path);
		(void) free((FREE_P *)dstk[dx]);
		dstk[dx] = NULL;
	/*
	 * Scan the directory.
	 */
		for (dp = readdir(dfp); dp; dp = readdir(dfp)) {

#if	defined(DHINS) || defined(sgi)
			if (dp->d_ino == 0 || dp->d_name[0] == '.')
#endif

#if	defined(eta10)
			if (dp->bsdd_ino == 0 || dp->bsdd_name[0] == '.')
#endif
				continue;
		/*
		 * Form the full path name and get its status.
		 */

#if	defined(DHINS) || defined(sgi)
			if ((nl = pl + dp->d_namlen) >= sizeof(path)) {
#endif

#if	defined(eta10)
			if ((nl = pl + dp->bsdd_namlen) >= sizeof(path)) {
#endif

				(void) fprintf(stderr,
					"%s: /dev entry name too long: %s\n",
					Pn,
					
#if	defined(DHINS) || defined(sgi)
					dp->d_name
#endif

#if	defined(eta10)
					dp->bsdd_name
#endif

					);
				exit(1);
			}
			(void) strncpy(&path[pl],

#if	defined(DHINS) || defined(sgi)
				dp->d_name, (STRNCPY_L)dp->d_namlen
#endif

#if	defined(eta10)
			      dp->bsdd_name, (STRNCPY_L)dp->bsdd_namlen
#endif
				      
				      );
			path[nl++] = '\0';
			if (lstat(path, &sb) != 0) {
				(void) fprintf(stderr,
					"%s: can't lstat %s: %s\n",
					Pn, path, sys_errlist[errno]);
				err = 1;
				continue;
			}
		/*
		 * If it's a subdirectory, stack its name for later
		 * processing.
		 */
			if ((sb.st_mode & S_IFMT) == S_IFDIR) {
				(void) stkdir(&dstk, &dn, &dx, path);
				continue;
			}
		/*
		 * Skip all but character devices.
		 */
			if ((sb.st_mode & S_IFMT) != S_IFCHR)
				continue;
		/*
		 * Make room for another Devtp[] entry.
		 */
			if (i >= Ndev) {
				Ndev += DEVINCR;
				if (Devtp == NULL)
				    Devtp = (struct dev *)malloc(
					(MALLOC_S)(sizeof(struct dev) * Ndev));
				else
				    Devtp = (struct dev *)realloc(
					(MALLOC_P *)Devtp,
					(MALLOC_S)(sizeof(struct dev)*Ndev));
				if (Devtp == NULL) {
					(void) fprintf(stderr,
						"%s: no space for devices\n",
						Pn);
					exit(1);
				}
			}
		/*
		 * Store the name and device number in the Devtp[] entry.
		 */
			Devtp[i].rdev = sb.st_rdev;
			if ((Devtp[i].name = (char *)malloc(nl)) == NULL) {
				(void) fprintf(stderr,
					"%s: no space for /dev/%s\n",
					Pn,
					
#if	defined(DHINS) || defined(sgi)
					dp->d_name
#endif

#if	defined(eta10)
					dp->bsdd_name
#endif
					
					);
				exit(1);
			}
			(void) strcpy(Devtp[i].name, path);

#if	defined(sgi) || defined(sun)
		/*
		 * Save the locations of clone devices.
		 */

# if	defined(sgi)
			if (major(Devtp[i].rdev) == USEMA_MAJOR
			||  major(Devtp[i].rdev) == CLN2_MAJOR)
#endif

# if	defined(sun)
			if (major(Devtp[i].rdev) == CLONEMAJ)
#endif

			{
			  if ((c = (struct clone *)malloc(sizeof(struct clone)))
			  == NULL) {
			    (void) fprintf(stderr,
				"%s: no space for clone device: %s\n",
				Pn, Devtp[i].name);
			    exit(1);
			  }
			  c->dp = &Devtp[i];
			  c->next = Clone;
			  Clone = c;
			}
#endif	/* sun */

			i++;
		}
		(void) closedir(dfp);
	}
/*
 * Free any directory stack space.
 */
	if (dstk != NULL)
		(void) free((FREE_P *)dstk);
/*
 * Reduce the Devtp[] table to its minimum size.
 */
	if (Ndev > i) {
		Ndev = i;
		Devtp = (struct dev *)realloc((MALLOC_P *)Devtp,
		      (MALLOC_S)(sizeof(struct dev) * Ndev));
	}
	if (err)
		exit(1);
/*
 * Allocate sorting pointers and sort them by Devtp[] device number.
 */
	if ((Sdev = (struct dev **)malloc((MALLOC_S)(sizeof(struct dev *)
	    * Ndev)))
	== NULL) {
		(void) fprintf(stderr, "%s: no space for device pointers\n",
			Pn);
		exit(1);
	}
	for (i = 0; i < Ndev; i++) {
		Sdev[i] = &Devtp[i];
	}
	(void) qsort((QSORT_P *)Sdev,(size_t)Ndev,(size_t)sizeof(struct dev *),
		compdev);
}


/*
 * stkdir() - stack directory name
 */

static void
stkdir(d, n, x, p)
	char ***d;		/* array of directory pointers */
	int *n;			/* number of pointers */
	int *x;			/* current index */
	char *p;		/* directory path */
{
	if (*d == NULL) {

	/*
	 * Allocate first entry.
	 */
		if ((*d = (char **)malloc(sizeof(char *))) == NULL) {

stkdir_nospace:

			(void) fprintf(stderr,
				"%s: no space for directory stack at %s\n",
				Pn, p);
			exit(1);
		}
		*n = 1;
		*x = 0;
	} else if (*x >= *n) {

	/*
	 * Allocate additional space as required.
	 */
		*n += 1;
		if ((*d = (char **)realloc((MALLOC_P *)*d,
		          (MALLOC_S)(*n * sizeof(char *))))
		== NULL)
			goto stkdir_nospace;
	}
/*
 * Allocate space for the name, copy it there and put its pointer on the stack.
 */
	if (((*d)[*x] = (char *)malloc((MALLOC_S)(strlen(p) + 1))) == NULL) {
		(void) fprintf(stderr, "%s: no space for %s\n", Pn, p);
		exit(1);
	}
	(void) strcpy((*d)[*x], p);
	*x += 1;
}
