/*
 *	lsof.h - header file for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */


#if	!defined(LSOF_H)
#define	LSOF_H	1

#include "machine.h"

#include <ctype.h>
#include <errno.h>
#include <pwd.h>
#include <setjmp.h>
#include <stdio.h>

#include <sys/stat.h>

# if	defined(DYNIX)
#include <mntent.h>
#include <strings.h>
#include <nlist.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <sys/dir.h>
#include <sys/domain.h>
#include <sys/file.h>
#include <sys/mbuf.h>
#include <sys/proc.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/un.h>
#include <sys/unpcb.h>
#include <sys/vfs.h>
#include <netinet/in.h>
#include <net/route.h>
#include <netinet/in_pcb.h>
#include <nfs/nfs.h>
#include <nfs/nfs_clnt.h>
#include <nfs/rnode.h>
#define KERNEL
#include <sys/file.h>
#include <sys/inode.h>
#include <ufs/mount.h>
#undef KERNEL
#include <sys/user.h>

#  if	!defined(DYNIX31)
/*
 * This is from sys/user.h.  It's ifdef KERNEL, though, and defining
 * KERNEL for user.h breaks other things.
 *
 * Open-file table extension object structure.
 */

struct  ofile_ext {
	struct  ofile_ext *oe_next;             /* list of unused entries */
	short           oe_nofile;              /* # ofile entries */
	short           oe_refcnt;              /* # proc's sharing this ext */
	short           oe_ccount;              /* # sharing proc's in memory */
	swblk_t         oe_swaddr;              /* disk address when swapped */
	sema_t          oe_mutex;               /* misc mutex */
	struct  ofile   *oe_ofile;              /* array of open files */
};
#  endif	/* ! DYNIX31 */
# endif	/* DYNIX */

# if	defined(eta10)
#include <bsd.h>
#include <dirent.h>
#include <malloc.h>
#include <mnttab.h>
#include <string.h>
#include <nlist.h>
#include <netinet/in.h>
#include <sys/conf.h>
#include <sys/file.h>
#include <sys/fcntl.h>
#include <sys/flock.h>
#include <sys/immu.h>
#include <sys/inode.h>
#include <sys/mount.h>
#include <sys/region.h>
#include <sys/proc.h>
#include <sys/stream.h>
#include <sys/time.h>
#include <sys/fs/s5dir.h>
#include <sys/user.h>
#include <sys/var.h>
#include <sys/fs/nfs/types.h>
#include <sys/fs/nfs/ucred.h>
#include <sys/fs/nfs/nfs.h>
#include <sys/fs/nfs/rnode.h>
#include <sys/net/domain.h>
#include <sys/net/protosw.h>
#include <sys/socket.h>
#include <sys/net/socketvar.h>
#include <netinet/in.h>
#include <net/route.h>
#include <netinet/in_pcb.h>

/*
 * This emulation of <ndir.h> avoids conflicts on the direct structure
 * and the DIRSIZ macro with <sys/fs/s5dir.h>.
 */

struct bsddirect {
	u_long bsdd_ino;		/* inode number of entry */
	u_short bsdd_reclen;		/* length of this record */
	u_short bsdd_namlen;		/* length of string in d_name */
	char bsdd_name[MAXNAMLEN + 1];	/* name must be no longer than this */
};

#define	readdir	readbsddir
extern struct bsddirect *readbsddir();

# endif	/* eta10 */

# if	defined(hpux)
#include <stdlib.h>
#include <dirent.h>
#include <mntent.h>
#include <string.h>
#include <nlist.h>
#include <sys/vnode.h>
#include <sys/domain.h>
#include <sys/file.h>
#include <sys/inode.h>
#include <sys/mbuf.h>

#  if	defined(HPUX8)
#include <sys/pstat.h>
#  endif

#include <sys/resource.h>
#include <sys/proc.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <netinet/in.h>
#include <net/route.h>
#include <netinet/in_pcb.h>
#include <rpc/types.h>
#include <nfs/nfs.h>
#include <nfs/nfs_clnt.h>
#include <nfs/rnode.h>
#include <nfs/snode.h>
#include <sys/un.h>
#include <sys/unpcb.h>
#include <sys/vfs.h>
#include <sys/vmmac.h>
#include <sys/user.h>

/*
 * This is from <sys/mount.h>, defined under the _KERNEL symbol.
 * Unfortunately, defining _KERNEL causes <sys/mount.h> to include
 * other header files not in <sys>.
 */

struct mount {
	struct	mount *m_hforw;  /* forward hash pointer */
	struct	mount *m_hback;  /* backward hash pointer */
	struct	mount *m_rhforw; /* forward hash pointer for real device */
	struct	mount *m_rhback; /* backward hash pointer for real device */
	struct vfs	*m_vfsp; /* vfs structure for this filesystem */
	dev_t	m_dev;		 /* device mounted */
   };
# endif	/* hpux */

# if	defined(_IBMR2)
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/domain.h>

#  if	!defined(_KERNEL)
#define _KERNEL	1
#  endif

#include <sys/file.h>
#undef	_KERNEL
#include <sys/mbuf.h>
#include <sys/mntctl.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/sysmacros.h>
#include <sys/un.h>
#include <sys/unpcb.h>
#include <netinet/in.h>
#include <net/route.h>
#include <netinet/in_pcb.h>
#include <sys/vattr.h>
#include <sys/vfs.h>
#include <sys/vmount.h>
#include <sys/vnode.h>
#include <procinfo.h>
#include <nfs/rnode.h>

#  if	_AIXV>=30200
#include <sys/specnode.h>
#  endif

/*
 * AIX doesn't supply cdrnode.h.
 */

struct cdrnode {
	caddr_t		f1[4];
	struct gnode	f2;
	dev_t		f3;
	ino_t		cn_inumber;	/* inode number */
	caddr_t		f4;
	cnt_t		f5[2];
	u_short		f6;
	uint		f7[2];
	uchar		f8[3];
	off_t		cn_size;	/* size of file in bytes */
};

/*
 * AIX doesn't supply ld_data.h.
 */

struct le {			/* ersatz loader_entry */
	struct le *next;	/* next entry pointer */
	ushort dummy1;
	ushort dummy2;
	uint dummy3;
	struct file *fp;	/* file table entry pointer */
};
# endif	/* _IBMR2 */

# if	defined(NeXT)
#include <c.h>
#include <stdlib.h>
#include <string.h>
#include <mntent.h>
#include <nlist.h>
#include <signal.h>

#  if	NS<300
#include <kern/queue.h>
#include <sys/policy.h>
#  endif	/* NS<300 */

#  if	!defined(NCPUS)
#define NCPUS	1
#  endif

#  if	NS<300
#include <kern/task.h>
#  endif	/* NS<300 */

#include <rpc/rpc.h>
#include <rpc/xdr.h>
#include <nfs/nfs.h>
#include <nfs/nfs_clnt.h>
#include <sys/vnode.h>
#include <nfs/rnode.h>
#include <sys/dir.h>
#include <sys/domain.h>

#  if	!defined(KERNEL)
#define KERNEL
#  endif

#include <sys/file.h>
#undef  KERNEL
#include <sys/mbuf.h>
#include <ufs/mount.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/stat.h>
#include <sys/ucred.h>
#include <netinet/in.h>
#include <net/route.h>
#include <netinet/in_pcb.h>
#include <sys/un.h>
#include <sys/unpcb.h>

#  if	!defined(SHOW_UTT)
#define SHOW_UTT
#  endif

#include <sys/user.h>
#undef	SHOW_UTT
#include <sys/proc.h>
#include <sys/vfs.h>
#include <ufs/inode.h>

/*
 * The following substitution compensates for the snode.h that NeXT does
 * not supply in NeXTStep 2.0 or 2.1.  The value of interest is s_realvp.
 */

struct snode {
	struct	snode *s_next;		/* must be first */
	struct	vnode s_vnode;		/* vnode associated with this snode */
	struct	vnode *s_realvp;	/* vnode for the fs entry (if any) */
};

#  if	NS>=300
/*
 *
 * The following substitution compensates for the <kern/task.h> that NeXT
 * doesn't supply in NeXTStep 3.0. 
 */

struct task {
	char		dummy[56];
	struct utask	*u_address;	/* user task structure address */
	struct proc	*proc;		/* corresponding process */
};
#  endif	/* NS>=300 */
# endif	/* NeXT */

# if	defined(sgi)
#include <rpc/types.h>
#include <malloc.h>
#include <mntent.h>
#include <string.h>
#include <nlist.h>
#include <netinet/in.h>
#include <sys/conf.h>
#include <sys/dir.h>
#define	_KERNEL
#include <sys/file.h>
#undef	_KERNEL
#include <sys/fcntl.h>
#include <sys/flock.h>
#include <sys/immu.h>
#include <sys/inode.h>
#include <sys/major.h>

# if	defined(MFREE)
#undef	MFREE
# endif

#include <sys/mbuf.h>

# if	defined(MFREE)
#undef	MFREE
# endif

#include <sys/mount.h>
#include <sys/proc.h>
#include <sys/stream.h>
#include <sys/syssgi.h>
#include <sys/un.h>
#include <sys/unpcb.h>
#include <sys/user.h>
#include <sys/region.h>
#include <sys/var.h>
#include <sys/fs/nfs.h>
#include <sys/fs/rnode.h>
#include <sys/domain.h>
#include <sys/sysmacros.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <netinet/in.h>
#include <net/route.h>
#include <netinet/in_pcb.h>
# endif	/* sgi */

# if	defined(sun)
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <kvm.h>
#include <mntent.h>
#include <nlist.h>
#include <signal.h>
#include <rpc/types.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <sys/stream.h>
#include <sys/un.h>
#include <sys/unpcb.h>
#include <netinet/in.h>
#include <net/route.h>
#include <netinet/in_pcb.h>
#include <sys/vfs.h>
#include <sys/vnode.h>
#include <nfs/nfs.h>
#include <nfs/rnode.h>
#include <specfs/snode.h>
#include <sys/domain.h>
#include <sys/mbuf.h>
#include <sys/proc.h>
#include <sys/sysmacros.h>
#include <sys/user.h>
#include <tmpfs/tmpnode.h>
#include <ufs/inode.h>
#include <ufs/mount.h>

#  if	!defined(KERNEL)
#define KERNEL	1
#  endif

#include <sys/file.h>
#include <specfs/fifonode.h>
#undef	KERNEL
# endif	/* sun */

extern int errno;

# if	defined(DYNIX) || defined(sun)
extern int nlist();
# endif

extern char *optarg;
extern int optind;
extern char *sys_errlist[];

#define DEVINCR		1024	/* device table malloc() increment */
#define	FDLEN		8	/* file descriptor printing array length */
#define IPROTOL		5	/* Internet protocol length */
#define LOGINML		9	/* login name length */
#define N_REG		0	/* node is regular */
#define N_BLK		1	/* node is a block device */
#define	N_CHR		2	/* node is a character device */
#define	N_COM		3	/* node is on an SGI "com" device */
#define N_FIFO		4	/* node is a FIFO */
#define N_NFS		5	/* node is an NFS file system */
#define N_STREAM	6	/* node is a stream */
#define N_TMP		7	/* node is a tmpfs */
#define N_MPC		8	/* node is a multiplexed device */
#define PIDINCR		10	/* PID table malloc() increment */
#define STATTMO		5	/* stat() timeout in seconds */
#define UIDINCR		10	/* UID table malloc() increment */
#define USERPRTL	8

#if	defined(HASSTREAMS)
#define	STRNML		32	/* stream name length (maximum) */
#endif

# if	defined(DYNIX)
#define	KA_T		off_t
#define MALLOC_P	char
#define FREE_P		MALLOC_P
#define MALLOC_S	unsigned
#define N_UNIX		"/dynix"

#  if	defined(i386)
#define	PTBITS		0xfff	/* 4096 byte pages */
# else
#define	PTBITS		0x1ff	/* 512 byte pages */
#  endif

#define QSORT_P		char
#define	READLEN_T	int
#define STRNCPY_L	int
#define SWAP		"/dev/drum"
#define	U_SIZE		sizeof(struct user)

#  if	defined(DYNIX31)
#define strchr		index
#define strrchr		rindex
#  endif

#  if	defined(SEQUENT)
#define strchr		index
#define strrchr		rindex
#define	uid_t		int
#define	unp_addr	unp_remaddr
#  endif
# endif	/* DYNIX */

# if	defined	(eta10)
#define	FSNAMEL		4
#define	KA_T		long
#define N_UNIX		"/unix"
#define MALLOC_P	char
#define	MNTTAB		"/etc/mnttab"
#define FREE_P		MALLOC_P
#define MALLOC_S	unsigned
#define	PROCSIZE	sizeof(struct proc)
#define QSORT_P		char
#define	READLEN_T	unsigned
#define STRNCPY_L	size_t
#define U_SIZE		sizeof(struct user)
# endif

# if	defined(hpux)
#define	KA_T		off_t
#define MALLOC_P	char
#define FREE_P		void
#define MALLOC_S	unsigned
#define MOUNTED		MNT_MNTTAB
#define N_UNIX		"/hp-ux"
#define PROCSIZE	sizeof(struct proc)
#define QSORT_P		void
#define	READLEN_T	int
#define STRNCPY_L	size_t
#define SWAP		"/dev/swap"

#  if	defined(HPUX7)
#define unp_addr	unp_locaddr
/*
 * HP-UX 7.x SWAP must be read in DEV_BSIZE chunks.
 */
#define U_SIZE		(((DEV_BSIZE+sizeof(struct user))/DEV_BSIZE)*DEV_BSIZE)
#  endif

#  if	defined(HPUX8)
#define	U_SIZE		sizeof(struct user)
#  endif
# endif	/* hpux */

# if	defined(_IBMR2)
#define	KA_T		off_t
#define MALLOC_P	void
#define FREE_P		MALLOC_P
#define MALLOC_S	size_t
#define PROCSIZE	sizeof(struct procinfo)
#define QSORT_P		void
#define	READLEN_T	int
#define STRNCPY_L	int
#define U_SIZE		sizeof(struct user)
# endif

# if	defined(NeXT)
#define	KA_T		off_t
#define MALLOC_P	void
#define FREE_P		MALLOC_P
#define MALLOC_S	size_t
#define PROCSIZE	sizeof(struct proc)
#define QSORT_P		void
#define	READLEN_T	int
#define STRNCPY_L	int
#define U_SIZE		sizeof(struct user)

#  if	!defined(VMUNIX)
#define VMUNIX		"/mach"
#  endif
# endif	/* NEXT */

# if	defined	(sgi)
#define	FSNAMEL		4
#define	KA_T		off_t
#define N_UNIX		"/unix"
#define MALLOC_P	void

#  if	defined(MNTTAB)
#undef	MNTTAB
#  endif

#define	MNTTAB		MOUNTED
#define FREE_P		MALLOC_P
#define MALLOC_S	size_t
#define	PROCSIZE	sizeof(struct proc)
#define QSORT_P		void
#define	READLEN_T	unsigned
#define STRNCPY_L	size_t
#define U_SIZE		sizeof(struct user)
# endif	/* sgi */

# if	defined(sun)
#define CLONEMAJ	37	/* major device number for clone device */
#define	KA_T		off_t
#define MALLOC_P	char
#define FREE_P		MALLOC_P
#define MALLOC_S	unsigned
#define N_UNIX		"/vmunix"
#define PROCSIZE	sizeof(struct proc)
#define QSORT_P		char
#define	READLEN_T	int
#define STRNCPY_L	int
#define U_SIZE		sizeof(struct user)
# endif


/*
 * NFS attribute redefinitions
 */

# if	defined(sgi)
#define	r_snfsattr	r_nfsattr
# endif


/*
 * Process structure redefinitions
 */

# if	defined(_IBMR2)
#define p_pid	pi_pid
#define p_stat	pi_stat
#define p_uid	pi_uid
# endif


/*
 * User structure redefinitions
 */

# if	defined(NeXT)
#define u_comm	uu_comm
#define u_cdir	uu_cdir
#define u_rdir	uu_rdir
# endif


/*
 * Global storage definitions (including their structure definitions)
 */

extern char Access;

# if	defined(sgi) || defined(sun)
struct clone {
	struct dev *dp;		/* pointer to device entry in Devtp[] */
	struct clone *next;

};
extern struct clone *Clone;
# endif	/* sgi || sun */

extern char *Command;
extern char *Core;
extern dev_t Dev;
extern char Devch[];

struct dev {
	dev_t rdev;		/* device */
	char *name;		/* name */

};
extern struct dev *Devtp;

extern char Fd[];
extern int Fdev;
extern int Fdevmsg;
extern int Fhelp;

#if	defined(_IBMR2)
extern int Flibdup;
#endif

extern int Fnet;
extern int Fnfs;
extern int Foffset;
extern int Fprint;
extern int Fsize;

# if	defined(eta10) || defined(sgi)
extern struct fsinfo *Fsinfo;
extern KA_T Ftaddr;			/* file table kernel address */
extern KA_T Ftaddrl;			/* file table kernel address limit */
# endif

extern int Fterse;
extern int Funix;
extern int Futol;
extern int Hdr;
extern long Inode;

extern char Iproto[];

# if	defined(sun)
extern kvm_t *Kd;
# endif

# if	defined(DEHINSG)
extern int Kmem;
# endif

# if	defined(HASVNODES)
struct l_vfs {

	struct vfs *addr;	/* kernel address */

# if	defined(DYNIX) || defined(hpux) || defined(_IBMR2)
	dev_t dev;		/* device */
# endif

	char *dir;		/* mounted directory */
	char *fsname;		/* file system name */

# if	defined(_IBMR2)
	int vmt_flags;		/* vmount flags */
	int vmt_gfstype;	/* vmount gfs type */
# endif

	struct l_vfs *next;	/* next entry pointer */
};
extern struct l_vfs *Lvfs;
# endif	/*HASVNODES */

extern char Lock;

# if	defined(eta10) || defined(HPUX7) || defined(_IBMR2)
extern int Mem;
# endif

struct mounts {
        dev_t dev;              /* st_dev */
        char *dir;              /* directory */
        char *fsname;           /* file system */
	ino_t inode;		/* st_ino */

# if	defined(DYNIX) || defined(eta10) || defined(_IBMR2) || defined(NeXT)
	u_short mode;		/* st_mode */
# endif

# if	defined(hpux) || defined(sgi) || defined(sun)
	mode_t mode;		/* st_mode */
# endif

        struct mounts *next;    /* forward link */
	dev_t rdev;		/* st_rdev */

};
extern struct mounts *Mtab;

extern int Mxpid;
extern int Mxuid;

#if	defined(HASSECURITY)
extern int Myuid;
#endif

extern char Namech[];
extern int Ndev;

# if	defined(eta10) || (sgi)
extern short Nfstyp;
# endif

extern struct nlist Nl[];

# if	defined(DYNIX)
#define X_PROC          0
#define X_NPROC         1
#define X_NFS_OPS	2
#define	X_SYSMAP	3
# endif

# if	defined(eta10)
#define	X_FILE		0
#define	X_FSINFO	1
#define	X_NFSTYP	2
#define X_PROC		3
#define	X_VAR		4

#  if	defined(LACHSOCK)
#define	X_SOCKDEV	5
#define	X_SOCKTAB	6
#  endif
# endif	/* eta10 */


# if	defined(hpux)

#  if	defined(hp9000s300)
#define X_PROC          0
#define X_NFS_OPS	1
#define X_NPROC         2
#define X_VN_FOPS	3

#   if	defined(HPUX7)
#define X_USRPTMAP	4
#define X_USRPT		5
#   endif
#  endif

#  if	defined(hp9000s800)
#define X_PROC          0
#define X_NPROC         1
#define X_NFS_OPS	2
#define X_VN_FOPS	3

#   if	defined(HPUX7)
#define X_UBASE		4
#define X_NPIDS		5
#   endif
#  endif
# endif	/* hpux */


# if	defined(NeXT)
#define X_ALLPROC	0
#define X_NFS_OPS	1
# endif

#if	defined(sgi)
#define	X_FILE		0
#define	X_FSINFO	1
#define	X_NFSTYP	2
#define	X_PROC		3
#define X_VAR		4
#endif

# if	defined(sun)
#define X_NFS_OPS	0
#define X_TMP_OPS	1
# endif

extern char *Nmlst;

# if	defined(DYNIX)
extern int Notkmem;
# endif

extern int Npid;
extern int Ntype;
extern int Nuid;
extern off_t Offset;
extern int Offset_def;
extern int Pid;
extern char *Pn;

# if	defined(HPUX7) && defined(hp9000s800)
extern int npids;
extern struct proc *proc;
# endif

# if	defined(hpux)
extern long Procaddr;
# endif

extern struct dev **Sdev;

struct sfile {
	char *name;		/* file name */
	char *devnm;		/* device name (optional) */
	dev_t dev;		/* device */

# if	defined(_IBMR2)
	chan_t ch;		/* channel (last path component, if numeric) */
# endif

	u_short mode;		/* S_IFMT mode bits from stat() */
	int type;		/* file type: 0 = file system
				 *	      1 = regular file */
	ino_t i;		/* inode number */
	int f;			/* file found flag */
	struct sfile *next;	/* link to next entry */

};
extern struct sfile *Sfile;

extern off_t Size;
extern int Size_def;
extern int *Spid;

# if	defined(LACHSOCK)
extern int sockdev;
extern KA_T Socktab;
# endif

extern int *Suid;

# if	defined(DYNIX) || defined(hpux)
extern int Swap;
# endif

# if	defined(DYNIX)
extern struct pte *Sysmap;
# endif

extern jmp_buf Tmo_jbuf;
extern char Type[];

extern char User[];

# if	defined(HPUX7) && defined(hp9000s800)
extern struct user *ubase;
#endif

# if	defined(HPUX7) && defined(hp9000s300)
extern struct pte *Usrptmap;
extern struct pte *usrpt;
# endif

# if	defined(eta10) || defined(sgi)
extern struct var Var;
# endif

extern char Version[];

# if	defined(hpux)
extern struct fileops *Vnfops;
# endif
#endif	/* LSOF_H */
