/*
 *	machine.h - machine-specific definitions for lsof
 */


/*
 * Copyright 1991, 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the author nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the author and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */


#include <sys/types.h>
#include <sys/param.h>


/*
 *======================================================================
 *
 * Operating system shorthand definitions
 *
 *
 * These single character values are used in the following operating
 * system shorthand definitions.
 *
 *	D	DYNIX
 *	E	eta10
 *	H	hpux
 *	I	_IBMR2
 *	N	NeXT
 *	S	sun
 *	SG	SGI
 *
 *======================================================================
 */


#if	defined(DYNIX31)
/*
 * DYNIX31 implies DYNIX and !SEQUENT.
 *
 * Note: this definition should come before any use of the DYNIX definition.
 */

#undef	SEQUENT
#define	DYNIX	1
#endif


#if     defined(DYNIX) || defined(eta10) || defined(hpux) || defined(_IBMR2) || defined(NeXT)
/*
 * DEHIN is defined for DYNIX or ETA10/ETAV or HP-UX or IBMR2/AIX or NeXT
 * systems.
 */

#define	DEHIN	1
#endif


#if     defined(DYNIX) || defined(eta10) || defined(hpux) || defined(_IBMR2) || defined(NeXT) || defined(sgi)
/*
 * DEHINSG is defined for DYNIX or ETA10/ETAV or HP-UX or IBMR2/AIX
 * or NeXT or SGI systems.
 */

#define	DEHINSG	1
#endif


#if     defined(DYNIX) || defined(eta10) || defined(hpux) || defined(NeXT) || defined(sun)
/*
 * DEHNS is defined for DYNIX or ETA10/ETAV or HP-UX or NeXT or Sun systems.
 */

#define	DEHNS	1
#endif


#if     defined(DYNIX) || defined(eta10) || defined(hpux) || defined(NeXT) || defined(sgi) || defined(sun)
/*
 * DEHNSSG is defined for DYNIX or ETA10/ETAV or HP-UX or NeXT or SGI or Sun
 * systems.
 */

#define	DEHNSSG	1
#endif


#if     defined(DYNIX) || defined(eta10) || defined(_IBMR2) || defined(NeXT) || defined(sgi) || defined(sun)
/*
 * DEINSSG is defined for DYNIX or ETA10/ETAV or IBMR2/AIX or NeXT
 * or SGI or Sun systems.
 */

#define	DEINSSG	1
#endif


#if     defined(DYNIX) || defined(eta10) || defined(NeXT) || defined(sgi) || defined(sun)
/*
 * DENSG is defined for DYNIX or ETA10/ETAV or NeXT or SGI or sun systems.
 */

#define	DENSSG	1
#endif


#if     defined(DYNIX) || defined(hpux) || defined(_IBMR2) || defined(NeXT)
/*
 * DHIN is defined for DYNIX or HP-UX or NeXT or NeXT systems.
 */

#define	DHIN	1
#endif



#if     defined(DYNIX) || defined(hpux) || defined(_IBMR2) || defined(NeXT) || defined(sun)
/*
 * DHINS is defined for DYNIX or HP-UX or IBMR2/AIX or NeXT or Sun systems.
 */

#define	DHINS	1
#endif


#if     defined(DYNIX) || defined(hpux) || defined(NeXT) || defined(sun)
/*
 * DHNS is defined for DYNIX or IBMR2/AIX or NeXT or Sun systems.
 */

#define	DHNS	1
#endif


#if     defined(DYNIX) || defined(_IBMR2) || defined(NeXT) || defined(sun)
/*
 * DINS is defined for DYNIX or IBMR2/AIX or NeXT or Sun systems.
 */

#define	DINS	1
#endif


#if     defined(eta10) || defined(hpux) || defined(_IBMR2) || defined(NeXT) || defined(sgi) || defined(sun)
/*
 * EHINSSG is defined for ETA10/ETAV or HP-UX or IBMR2/AIX or NeXT or SGI or
 * Sun systems.
 */

#define	EHINSSG	1
#endif


#if     defined(hpux) || defined(_IBMR2) || defined(NeXT) || defined(sun)
/*
 * HINS is defined for HP-UX or IBMR2/AIX or NeXT or Sun systems.
 */

#define	HINS	1
#endif


/*
 *=======================================================================
 *
 * Some of the following definitions use the operating system shorthand
 * definitions.
 *
 *=======================================================================
 */


#if	defined(DHINS)
/*
 * BSDSOCK is defined for those operating systems that use the BSD
 * socket model.
 */

#define	BSDSOCK		1
#endif


#if	defined(DHINS)
/*
 * HASFTYPE is defined for those operating systems that have a file type
 * element, f_type, in the file structure.
 */

#define	HASFTYPE	1
#endif


#if     defined(DHNS)
/*
 * HASKOPT is defined for those systems that support the -k option of
 * reading the kernel's name list from an optional file.
 */

#define	HASKOPT	1
#endif


/*
 * Define HASSECURITY to restrict the listing of all open files to the
 * root user.  When HASSECURITY is defined, the non-root user may list
 * only files whose processes have the same user ID as the real user ID
 * (the one that its user logged on with) of the lsof process.
 */

/* #define	HASSECURITY	1	/* enable security mode */


#if	defined(eta10) || defined(sgi) || defined(sun)
/*
 * HASSTREAMS is defined for those systems that support streams.
 */

#define	HASSTREAMS	1
#endif


#if	defined(DHINS)
/*
 * HASVNODES is defined for those operating systems that use the Sun
 * virtual file system node, the vnode.  BSD derivatives usually do;
 * System V derivatives usually don't.
 */

#define	HASVNODES	1
#endif


#if	defined(eta10)
/*
 * LACHSOCK is defined for System V operating systems that use sockets
 * from the Lachman Associates, Inc. System V Streams TCP/IP, Release 3.0.
 */

#define	LACHSOCK	1
#endif


#if	defined(NeXT) && !defined(MACH)
/*
 * The definition of MACH for the NeXT is required for proper header
 * file configuration -- i. e., some header files have ``#ifdef MACH''
 * statements that affect the size of kernel structures.
 */

#define MACH	1
#endif
