/*
 *	main.c - main function for lsof
 */


/*
 *	Lsof lists files opened by processes of:
 *
 *								      Shorthand
 *									   Code
 *	AIX 3.1.[357]			(#define _IBMR2)		      I
 *	AIX 3.2				(#define _IBMR2, #define _AIXV=30200) I
 *	Purdue DYNIX 3.0.12		(#define DYNIX)			      D
 *	Sequent DYNIX 3.0.12		(#define DYNIX, #define SEQUENT)      D
 *	Sequent DYNIX 3.1.2		(#define DYNIX, #define DYNIX31)      D
 *	ETA-10 ETAV			(#define eta10)			      E
 *	HP-UX 7.x			(#define hpux, #define HPUX7)	      H
 *	HP-UX 8.x			(#define hpux, #define HPUX8)	      H
 *	NeXTStep 2.[01]			(#define NeXT)			      N
 *	NeXTStep 3.0			(#define NeXT, #define NS=300)	      N
 *	Silicon Graphics IRIX 4.0.5	(#define sgi)			      SG
 *	SunOS 4.1.1			(#define sun)			      S
 *
 *	See "machine.h" for #defines that combine the above options, using
 *	shorthand codes.
 */

/*
 * Copyright 1991, 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the author nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the author and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#if	!defined(lint)
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: main.c,v 1.12 92/10/09 10:27:31 abe Exp $";
#endif

char Version[] = "2.06";

#include "lsof.h"
#include "proto.h"


/*
 * main() - main program
 */

main(argc, argv)
	int argc;
	char *argv[];
{
	int c;
	int err = 0;
	char *fnm;
	int fprint;
	char *fsnm;
	int ftype;
	int i;
	int llen, llenx;
	struct mounts *mp;
	char *path;
	struct stat sb;
	int selflg, selpid, seluid;
	struct sfile *sfp;
	char syml[MAXPATHLEN];
	char *symlp;
	char symlx[MAXPATHLEN];

#if	defined(DYNIX)
	long kp;
	int nf, np;
	static struct ofile *ofp = NULL;
	struct proc *p;
	KA_T pa;
	struct proc ps;
	int px;
	struct user *u;
	struct ofile *uf;
	unsigned ui;
	struct user us;
	static int xnf = 0;

# if	!defined(DYNIX31)
	struct ofile_ext ofx;
# endif

# if	defined(DYNIX31)
	struct ofile_tab oft;
# endif
#endif	/* DYNIX */

#if	defined(eta10)
	long kp;
	struct proc *p;
	KA_T pa;
	struct proc ps;
	int px;
	user_t *u;
#endif

#if	defined(hpux)

# if	defined(HPUX7)
	int k;
	char us[U_SIZE];	/* must read HP-UX SWAP in DEV_BSIZE chunks */
# endif

# if	defined(HPUX8)
	struct ofile_t *ofp;
	struct ofile_t oft;
	struct user us;
# endif

	int j;
	long kp;
	int np;
	struct proc *p;
	struct proc ps;
	int px;
	struct user *u;
#endif	/* hpux */

#if	defined(_IBMR2)
	int np;
	struct procinfo *p;
	struct user *u;
	struct user us;
#endif

#if	defined(NeXT)
	MALLOC_S nb;
	int nf;
	struct proc *np, *p, *procp, ps;
	static MALLOC_S pnb = 0;
        struct task t;
        struct file **uf = NULL;
        struct utask *u, ut;
#endif

#if	defined(sgi)
	struct file **fp;
	long kp;
	int nf;
	struct proc *p;
	KA_T pa;
	struct proc ps;
	int px;
	struct user *u;
	char *uu;
	MALLOC_S uul;
#endif

#if	defined(sun)
	int nf;
	struct proc *p;
	struct user *u;
	struct file **uf;
	unsigned ui;
	static int xnf = 0;
	static struct file **xuf = NULL;
#endif

/*
 * Save program name for messages.
 */
	if ((Pn = strrchr(argv[0], '/')) != NULL)
		Pn++;
	else
		Pn = argv[0];

#if	defined(HASSECURITY)
/*
 * Set up for HASSECURITY option.
 */
	Myuid = (int)getuid();
#endif	/* HASSECURITY */

/*
 * Process arguments.
 */

#if	defined(HASKOPT)
	while ((c = getopt(argc, argv, "c:dhk:lLnNop:stu:U")) != EOF)
#else
	while ((c = getopt(argc, argv, "c:dhlLnNop:stu:U")) != EOF)
#endif
	{

		switch(c) {

		case 'c':
			Core = optarg;
			break;
		case 'd':
			Fdevmsg = 1;
			break;
		case 'h':
			Fhelp = 1;
			break;

#if	defined(HASKOPT)
		case 'k':
			Nmlst = optarg;
			break;
#endif

		case 'l':
			Futol = 0;
			break;

#if	defined(_IBMR2)
		case 'L':
			Flibdup = 0;
			break;
#endif

		case 'n':
			Fnet = 1;
			break;
		case 'N':
			Fnfs = 1;
			break;
		case 'o':
			Foffset = 1;
			break;
		case 'p':
			if (entpid(optarg))
				err = 1;
			break;
		case 's':
			Fsize = 0;
			break;
		case 't':
			Fterse = 1;
			break;
		case 'u':
			if (entuid(optarg))
				err = 1;
			break;
		case 'U':
			Funix = 1;
			break;
		case '?':
			err = 1;
			break;
		default:
			(void) fprintf(stderr, "%s: unknown option (%c)\n",
				Pn, c);
			err = 1;
		}
	}
	if (err || Fhelp)
		usage(err);

/*
 * Read the mount table and process any file names.
 */
	if ( ! readmnt())
		err = 1;
	for (i = optind; i < argc; i++) {

	/*
	 * Dereference a symbolic link.
	 */
		if ((llen = readlink(argv[i], syml, sizeof(syml) - 1)) >= 0) {
			syml[llen] = '\0';
			if (argv[i][0] == '/' && syml[0] != '/') {
				llenx = (strrchr(argv[i], '/') - argv[i]) + 1;
				if ((llenx + llen + 1) > sizeof(symlx)) {
					(void) fprintf(stderr,
					    "%s: sym link for %s too long\n",
					    Pn, argv[i]);
					exit(1);
				}
				(void) strncpy(symlx, argv[i],
					(STRNCPY_L)llenx);
				(void) strcpy(&symlx[llenx], syml);
				symlp = symlx;
				llen += llenx;
			} else
				symlp = syml;
			if ((path = (char *)malloc((MALLOC_S)(llen + 1)))
			== NULL) {
				(void) fprintf(stderr,
					"%s: no space for sym link of %s\n",
					Pn, argv[i]);
				exit(1);
			}
			(void) strcpy(path, symlp);
		} else
			path = argv[i];
	/*
	 * Check for file system argument.
	 */
		for (ftype = 1, mp = Mtab; mp; mp = mp->next) {
			if (strcmp(mp->dir, path) == 0) {
				ftype = 0;
				fnm = path;
				fsnm = mp->fsname;
				break;
			}
			if (strcmp(mp->fsname, path) == 0) {
				ftype = 0;
				fnm = mp->dir;
				fsnm = path;
				break;
			}
		}
		if (ftype) {
			fnm = path;
			fsnm = NULL;
		} else {
			sb.st_dev = mp->dev;
			sb.st_ino = mp->inode;
			sb.st_mode = mp->mode;
			sb.st_rdev = mp->rdev;
		}
	/*
	 * Stat the argument to obtain its mode and device.
	 */

		if (ftype && statsafely(fnm, &sb, STATTMO) != 0) {
			(void) fprintf(stderr, "%s: status error on %s: %s\n",
				Pn, fnm, sys_errlist[errno]);
			err = 1;
			continue;
		}
	/*
	 * Allocate an sfile structure and fill in the type, inode,
	 * find-flag and linkages.
	 */
		if ((sfp = (struct sfile *)malloc(sizeof(struct sfile)))
		== NULL) {
			(void) fprintf(stderr, "%s: no space for files\n", Pn);
			exit(1);
		}
		sfp->next = Sfile;
		Sfile = sfp;
		sfp->type = ftype;
		sfp->i = sb.st_ino;
		sfp->f = 0;
	/*
	 * Store the file name and file system name pointers in the sfile
	 * structure, allocating space as necessary.
	 */
		if (fnm == NULL || fnm == path)
			sfp->name = fnm;
		else {
		    if ((sfp->name = (char *)malloc((MALLOC_S)(strlen(fnm)+1)))
		    == NULL) {
			(void) fprintf(stderr,
			    "%s: no space for file name %s\n", Pn, fnm);
			exit(1);
		    }
		    (void) strcpy(sfp->name, fnm);
		}
		if (fsnm == NULL || fsnm == path)
			sfp->devnm = fsnm;
		else {
		    if ((sfp->devnm=(char *)malloc((MALLOC_S)(strlen(fsnm)+1)))
		    == NULL) {
			(void) fprintf(stderr,
			    "%s: no space for file system name %s\n", Pn, fsnm);
			exit(1);
		    }
		    (void) strcpy(sfp->devnm, fsnm);
		}
	/*
	 * Save the stat() buffer mode value in the sfile structure.
	 * Use st_rdev if the mode value is S_IFBLK or S_IFCHR; otherwise
	 * use st_dev.
	 */
		sfp->mode = sb.st_mode & S_IFMT;
		if (sfp->mode == S_IFBLK || sfp->mode == S_IFCHR)
			sfp->dev = sb.st_rdev;
		else
			sfp->dev = sb.st_dev;

#if	defined(_IBMR2)
	/*
	 * Save a (possible) AIX multiplexed channel number.
	 */
		sfp->ch = getchan(path);
#endif

	}
	if (err)
		usage(err);
/*
 * Initiate kernel memory accesses.
 */

#if	defined(DEHINSG)
	if ((Kmem = open(Core ? Core : "/dev/kmem", O_RDONLY, 0)) < 0) {
		(void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
			Core ? Core : "/dev/kmem", sys_errlist[errno]);
		err = 1;
	}
#endif

#if	defined(eta10) || defined(sgi)
        if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) == -1) {
		(void) fprintf(stderr,
			"%s: can't read kernel name list from %s\n",
			Pn, Nmlst ? Nmlst : N_UNIX);
		err = 1;
	} else {
		if ((kp = Nl[X_PROC].n_value) == NULL
		||  Nl[X_VAR].n_value == NULL
		||  kread((KA_T)Nl[X_VAR].n_value, (char *)&Var, sizeof(Var))) {
			(void) fprintf(stderr,
				"%s: can't read proc table info\n", Pn);
			err = 1;
		}
		if ((Ftaddr = (KA_T)Nl[X_FILE].n_value) == NULL) {
			(void) fprintf(stderr, "%s: no file table address\n",
				Pn);
			err = 1;
		}

# if	defined(eta10)
		Ftaddrl = Ftaddr + (Var.v_file * sizeof(struct file));
# endif

# if	defined(sgi)
		Ftaddr &= 0x7fffffff;
		Ftaddrl = Ftaddr + (Var.v_file * sizeof(struct file));
		kp &= 0x7fffffff;
		uul = (MALLOC_S) (sizeof(struct user)
		    +  (Var.v_nofiles * sizeof(struct file *)));
		if ((uu = malloc(uul)) == NULL) {
			(void) fprintf(stderr,
				"%s: no space for uarea and file pointers\n",
				Pn);
		err = 1;
		}
# endif	/* sgi */

# if	defined(LACHSOCK)
		if (Nl[X_SOCKDEV].n_value  == NULL
		||  kread((KA_T)Nl[X_SOCKDEV].n_value, (char *)&sockdev,
		    sizeof(sockdev)))
		{
			(void) fprintf(stderr,
				"%s: can't read socket device (%#x)\n",
				Pn, Nl[X_SOCKDEV].n_value);
			err = 1;
		}
		if ((Socktab = (KA_T)Nl[X_SOCKTAB].n_value) == NULL) {
			(void) fprintf(stderr, "%s: no socktab[] address\n",
				Pn);
			err = 1;
		}
# endif

	}
#endif	/* eta10 || sgi */

#if	defined(eta10) || defined(HPUX7) || defined(_IBMR2)
	if ((Mem = open("/dev/mem", O_RDONLY, 0)) < 0) {
		(void) fprintf(stderr, "%s: can't open /dev/mem: %s",
			Pn, sys_errlist[errno]);
		err = 1;
	}
#endif

#if	defined(DYNIX) || defined(HPUX7)
	if (Core == NULL || strcmp(Core, "/dev/kmem") == 0) {
		if ((Swap = open(SWAP, O_RDONLY, 0)) < 0) {
			(void) fprintf(stderr, "%s: %s: %s\n",
				Pn, SWAP, sys_errlist[errno]);
			err = 1;
		}
	}
#endif

#if	defined(_IBMR2)

/*
 * Read the user area base address.
 */
	if (knlist(&Nl, 1, sizeof(struct nlist)) ||  Nl[0].n_value == NULL) {
		(void) fprintf(stderr, "%s: can't get user area address\n");
		err = 1;
	}
#endif

#if	defined(NeXT)
        if (nlist(Nmlst ? Nmlst : VMUNIX, Nl) < 0) {
                (void) fprintf(stderr, "%s: can't read namelist from %s\n",
			Pn, Nmlst ? Nmlst : VMUNIX);
                err = 1;
        } else {
	    if (Nl[X_ALLPROC].n_value == 0l) {
		(void) fprintf(stderr, "%s: no address for %s\n",
			Pn, Nl[X_ALLPROC].n_un.n_name);
		err = 1;
	    } else {
		if (kread((KA_T)Nl[X_ALLPROC].n_value, (char *)&procp,
			  sizeof(procp)))
		{
		    (void) fprintf(stderr,
			"%s: can't read nl[X_ALLPROC].n_un.n_name: %s\n",
			Pn, sys_errlist[errno]);
		    err = 1;
		}
	    }
	}
#endif

#if	defined(sun)
	if ((Kd = kvm_open(Nmlst, Core, NULL, O_RDONLY, NULL)) == NULL) {
		(void) fprintf(stderr,
			"%s: kvm_open (namelist=%s, core=%s): %s\n", Pn,
			Nmlst ? Nmlst : "default",
			Core ? Core : "default",
			sys_errlist[errno]);
		err = 1;
	} else {
		if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
			(void) fprintf(stderr,
				"%s: can't read namelist from %s\n",
				Pn, Nmlst ? Nmlst : N_UNIX);
			err = 1;
		}
	}
#endif
	if (err)
		usage(err);
/*
 * Read the names, modes and device types of everything in /dev.
 *
 * Read the file system information table, if there is one.
 */
	(void) readdev();

#if	defined(eta10) || defined(sgi)
	if ( ! readfsinfo())
		usage(1);
#endif

/*
 * Read the process table.
 */

#if	defined(DYNIX) || defined(hpux)
	if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
                (void) fprintf(stderr, "%s: can't read namelist from %s\n",
			Pn, Nmlst ? Nmlst : N_UNIX);
                exit(1);
	}
	if (Nl[X_PROC].n_value == NULL
	||  kread((KA_T)Nl[X_PROC].n_value, (char *)&kp, sizeof(kp))
	||  Nl[X_NPROC].n_value == NULL
	||  kread((KA_T)Nl[X_NPROC].n_value, (char *)&np, sizeof(np))
	||  kp == NULL || np == 0) {
		(void) fprintf(stderr, "%s: can't read proc table info\n",
			Pn);
		exit(1);
	}

# if	defined(DYNIX)
/*
 * If the DYNIX Core file isn't /dev/kmem, prepare for kernel virtual
 * to physical address mapping in the core file.
 */
	if (Core != NULL && strcmp(Core, "/dev/kmem") != 0) {
		if (Nl[X_SYSMAP].n_value == NULL
		||  kread((KA_T)Nl[X_SYSMAP].n_value, (char *)&Sysmap,
			  sizeof(Sysmap))
		||  Sysmap == NULL)
		{
		    (void) fprintf(stderr,
			"%s: no _Sysmap for virtual to physical mapping\n",
				Pn);
			exit(1);
		}
		Notkmem = 1;		/* can't set until _Sysmap available */
	}
# endif

# if	defined(hpux)
	Vnfops = (struct fileops *) Nl[X_VN_FOPS].n_value;
# endif
#endif	/* DYNIX || hpux */

#if	defined(HPUX7) && defined(hp9000s300)
	if ((Usrptmap = (struct pte *)Nl[X_USRPTMAP].n_value) == NULL) {
		(void) fprintf(stderr, "%s: can't get kernel's Usrptmap\n", Pn);
		exit(1);
	}
	if ((usrpt = (struct pte *)Nl[X_USRPT].n_value) == NULL) {
		(void) fprintf(stderr, "%s: can't get kernel's usrpt\n", Pn);
		exit(1);
	}
#endif

#if	defined(HPUX7) && defined(hp9000s800)
	proc = (struct proc *)kp;
	if ((ubase = (struct user *)Nl[X_UBASE].n_value) == NULL) {
		(void) fprintf(stderr, "%s: can't get kernel's ubase\n", Pn);
		exit(1);
	}
	if (Nl[X_NPIDS].n_value == NULL
	||  kread((KA_T)Nl[X_NPIDS].n_value, (char *)&npids, sizeof(npids))) {
		(void) fprintf(stderr, "%s: can't get kernel's npids\n", Pn);
		exit(1);
	}
#endif

#if	defined(_IBMR2)
	if ((p = (struct procinfo *)malloc((size_t)PROCSIZE)) == NULL) {
		(void) fprintf(stderr,
			"%s: can't allocate space for 1 procinfo\n", Pn);
		exit(1);
	}
	np = 0;
	while (((np = getproc(p, np, PROCSIZE)) == -1) && errno == ENOSPC) {
		np = p->p_pid + 10;
		if ((p = (struct procinfo *)realloc((MALLOC_P *)p,
		    (size_t) (np * PROCSIZE))) == NULL) {
			(void) fprintf(stderr,
				"%s: no space for %ld procinfo's\n", Pn, np);
			exit(1);
		}
	}
#endif

#if	defined(sun)
	(void) kvm_setproc(Kd);
#endif

/*
 * Loop through the process table entries.
 */
	selflg = (Fnet || Fnfs || Funix || Npid || Nuid || Sfile) ? 0 : 1;
	selpid = (Fnet || Fnfs || Funix || Nuid || Sfile) ? 0 : 1;
	seluid = (Fnet || Fnfs || Funix || Sfile) ? 0 : 1;

#if	defined(DYNIX)
	for (p = &ps, px = 0, u = &us; px < np; px++)
#endif

#if	defined(eta10)
	for (p = &ps, px = 0; px < Var.v_proc; px++)
#endif

#if	defined(HPUX7)
	for (p = &ps, px = 0, u = (struct user *)us; px < np; px++)
#endif

#if	defined(HPUX8)
	for (p = &ps, px = 0, u = &us; px < np; px++)
#endif

#if	defined(_IBMR2)
	for (; np; np--, p++)
#endif

#if	defined(NeXT)
	for (np = procp, p = &ps; np; np = ps.p_nxt)
#endif

#if	defined(sgi)
	fp = (struct file **)(uu + sizeof(struct user));
	u = (struct user *)uu;
	for (p = &ps, px = 0; px < Var.v_proc; px++)
#endif

#if	defined(sun)
	while ((p = kvm_nextproc(Kd)) != NULL)
#endif

	{

#if	defined(DYNIX) || defined(eta10) || defined(sgi)
		pa = (KA_T)(kp + (long)(px * sizeof(struct proc)));
		if (kread(pa, (char *)&ps, sizeof(ps)))
			continue;
#endif

#if	defined(hpux)
		Procaddr = kp + (long)(px * sizeof(struct proc));
		if (kread((KA_T)Procaddr, (char *)&ps, sizeof(ps)))
			continue;
#endif

#if	defined(NeXT)
                if (kread((KA_T)np, (char *)&ps, sizeof(ps))) {
                        (void) fprintf(stderr,
                                "%s: can't read proc struct at %#x\n",
                                Pn, np);
                        exit(1);
                }
#endif

		if (p->p_stat == 0 || p->p_stat == SZOMB)
			continue;

#if	defined(HASSECURITY)
		if (Myuid && p->p_uid != Myuid)
			continue;
#endif

		fprint = selflg;
		for (i = 0; fprint == 0 && i < Npid; i++) {
			if (Spid[i] == p->p_pid)
				fprint = 1;
		}
		if (selpid && fprint == 0)
			continue;

#if	defined(NeXT)
        /*
         * Read the task associated with the process, and the user
         * area assocated with the task.
         */
                if (kread((KA_T) p->task, &t, sizeof(t)))
                        continue;
                if ((struct proc *)t.proc != np)
                        continue;
                if (kread((KA_T) t.u_address, &ut, sizeof(ut)))
                        continue;
                if ((struct proc *)ut.uu_procp != np)
                        continue;
		u = &ut;
#endif

/*
 * Check for processes owned by specified user(s).
 */
		for (i = 0; fprint == 0 && i < Nuid; i++) {
			if (Suid[i] == p->p_uid)
				fprint = 1;
		}
		if (seluid && fprint == 0)
			continue;

#if	defined(DYNIX) || defined(hpux) || defined(_IBMR2) || defined(sgi) || defined(sun)
	/*
	 * Get the user area associated with the process.
	 */
#endif

#if	defined(DYNIX)
		if (getu(p, &us))
			continue;
#endif

#if	defined(eta10)
		if ((u = getu(p, pa)) == NULL)
			continue;
#endif

#if	defined(hpux)
		if (getu(p, u))
			continue;
#endif

#if	defined(_IBMR2)
		u = &us;
		if (getuser(p, PROCSIZE, u, U_SIZE) != 0)
			continue;
#endif

#if	defined(NeXT)
/*
 * The user area is already available from reading the task area.
 */
#endif

#if	defined(sgi)
		if ((i = syssgi(SGI_RDUBLK, p->p_pid, uu, uul))
		< sizeof(struct user))
			continue;
		if (i <= sizeof(struct user)
		||  ((long)u->u_ofile - UADDR) != sizeof(struct user))
			nf = 0;
		else
			nf = (i - sizeof(struct user)) / sizeof(struct file *);
#endif

#if	defined(sun)
		if ((u = kvm_getu(Kd, p)) == NULL)
			continue;
#endif


	/*
	 * Print information on the current directory.
	 */
		Pid = p->p_pid;

#if	defined(DEINS) || defined(HPUX7) || defined(sgi)
		if (u->u_cdir)
#endif

#if	defined(HPUX8)
		if (p->p_cdir)
#endif

		{
			printprep(u->u_comm, " cwd", fprint, p->p_uid);

#if	defined(DINS) || defined(HPUX7)
			printnode((caddr_t)u->u_cdir);
#endif

#if	defined(eta10) || defined(sgi)
			printnode(u->u_cdir);
#endif

#if	defined(HPUX8)
			printnode((caddr_t)p->p_cdir);
#endif
		}
	/*
	 * Print information on the parent directory.
	 */

#if	defined(eta10) || defined(_IBMR2) || defined(sgi)
		if (u->u_pdir != NULL) {
			printprep(u->u_comm, "  pd", fprint, p->p_uid);

# if	defined(eta10) || defined(sgi)
			printnode(u->u_pdir);
# endif

# if	defined(_IBMR2)
			printnode((caddr_t)u->u_pdir);
# endif

		}
#endif	/* eta10 || _IBMR2 || sgi */

	/*
	 * Print information on the root directory.
	 */

		if (u->u_rdir != NULL) {
			printprep(u->u_comm, " rtd", fprint, p->p_uid);

#if	defined(eta10) || defined(sgi)
			printnode(u->u_rdir);
#endif

#if	defined(DHINS)
			printnode((caddr_t)u->u_rdir);
#endif

		}
	/*
	 * Print information on the text file.
	 */

#if	defined(_IBMR2)
		printtext(fprint, u, p->p_uid, p->pi_adspace);
#endif

#if	defined(eta10)
		if (u->u_exdata.ip != NULL) {
			printprep(u->u_comm, " txt", fprint, p->p_uid);
			printnode(u->u_exdata.ip);
		}
#endif	/* eta10 */

	/*
	 * Loop through user's files.
	 */

#if	defined(DYNIX)

# if	!defined(DYNIX31)
	/*
	 * Under DYNIX, the file pointers may be located in the user
	 * structure or in an extension to it.  Determine which is the
	 * case and read the pointers.
	 */

	if (u->u_ofile_ext) {
		if (kread((KA_T)u->u_ofile_ext, (char *)&ofx, sizeof(ofx)))
			nf = 0;
		else {
			nf = ofx.oe_nofile;
			ui = (unsigned)(ofx.oe_nofile * sizeof(struct ofile));
			if (ofp == NULL) {
				xnf = nf;
				if ((ofp = (struct ofile *)malloc(ui))
				== NULL) {
				     (void) fprintf(stderr,
					"%s: PID %d, no malloc ofile space\n",
					Pn, Pid);
				    exit(1);
				}
			} else if (nf > xnf) {
				xnf = nf;
				if ((ofp = (struct ofile *)realloc(
					   (MALLOC_P *)ofp, ui))
				== NULL) {
				    (void) fprintf(stderr,
					"%s: PID %d,no realloc ofile space\n",
					Pn, Pid);
				    exit(1);
				}
			}
			if (kread((KA_T)ofx.oe_ofile, (char *)ofp, (int)ui))
				nf = 0;
			else {
				nf = ofx.oe_nofile;
				uf = ofp;
			}
		}
	} else {
		nf = NOFILE;
		uf = u->u_lofile;
	}
# endif	/* ! DYNIX31 */

# if	defined(DYNIX31)
	/*
	 * Under DYNIX 3.1, the file pointers should be located in an extension
	 * to the user structure.  Determine if is the case and read the
	 * pointers.
	 */

	if (u->u_ofile_tab) {
		if (kread((KA_T)u->u_ofile_tab, (char *)&oft, sizeof(oft)))
			nf = 0;
		else {
			nf = oft.oft_nofile;
			ui = (unsigned)(oft.oft_nofile * sizeof(struct ofile));
			if (ofp == NULL) {
				xnf = nf;
				if ((ofp = (struct ofile *)malloc(ui))
				== NULL) {
				    (void) fprintf(stderr,
					"%s: PID %d, no malloc ofile space\n",
					Pn, Pid);
				    exit(1);
				}
			} else if (nf > xnf) {
				xnf = nf;
				if ((ofp = (struct ofile *)realloc(
					   (MALLOC_P *)ofp, ui))
				== NULL) {
				    (void) fprintf(stderr,
					"%s: PID %d, no realloc ofile space\n",
					Pn, Pid);
				    exit(1);
				}
			}
			if (kread((KA_T)oft.oft_ofile, (char *)ofp, (int)ui))
				nf = 0;
			else {
				nf = oft.oft_nofile;
				uf = ofp;
			}
		}
	} else {
		nf = NOFILE;
		uf = u->u_lofileXXX;
	}
# endif	/* DYNIX31 */
#endif	/* DYNIX */

#if	defined(NeXT)
	/*
	 * NeXTStep 2.0 and 2.1 file pointers come from a structure whose
	 * pointer is in the user area.
	 */
		nf = ut.uu_ofile_cnt;
                nb = (MALLOC_S) (sizeof(struct file) * nf);
		if (uf == NULL) {
			if ((uf = (struct file **)malloc(nb)) == NULL) {
				(void) fprintf(stderr,
					"%s: Pid %d, no file table space\n",
					Pn, Pid);
				exit(1);
			}
			pnb = nb;
		} else if (nb > pnb) {
			if ((uf = (struct file **)realloc((MALLOC_P *)uf, nb))
			== NULL) {
				(void) fprintf(stderr,
					"%s: PID %d, realloc file[]\n",
					Pn, Pid);
				exit(1);
			}
			pnb = nb;
		}
                if (kread((KA_T)ut.uu_ofile, (char *)uf, nb))
                        continue;
#endif	/* NeXT */
	
#if	defined(sun)
	/*
	 * Under SunOS 4.1.1, the file pointers may be located in the
	 * user structure or in a separately allocated area.  Determine
	 * which is the case and read the pointers.
	 */
		if ((caddr_t)u->u_ofile
		== ((caddr_t)&u->u_ofile_arr[0] - (caddr_t)&u
		+ (caddr_t)p->p_uarea)) {
			nf = NOFILE_IN_U;
			uf = &u->u_ofile_arr[0];
		} else {
			nf = u->u_lastfile + 1;
			ui = nf * sizeof(struct file);
			if (xuf == NULL) {
				xnf = nf;
				if ((xuf = (struct file **)malloc(ui))
				== NULL) {
					(void) fprintf(stderr,
						"%s: PID %d, malloc file[]\n",
						Pn, Pid);
					exit(1);
				}
			} else if (nf > xnf) {
				xnf = nf;
				if ((xuf = (struct file **)realloc(
					   (MALLOC_P *)xuf, ui))
				== NULL) {
					(void) fprintf(stderr,
						"%s: PID %d, realloc file[]\n",
						Pn, Pid);
					exit(1);
				}
			}
			if (kread((KA_T)u->u_ofile, (char *)xuf, (int)ui))
				nf = 0;
			uf = xuf;
		}
#endif	/* sun */

#if	defined(DYNIX)
		for (i = 0; i < nf; i++) {
			if (uf[i].of_file)
#endif

#if	defined(eta10)
		for (i = 0; i < Var.v_nofiles; i++) {
			if (u->u_ofile[i])
#endif

#if	defined(HPUX7)
		for (i = j = k = 0;; i++) {
			if (j >= SFDCHUNK) {

			/*
			 * Get next HP-UX file pointer "chunk".
			 */
			    while (++k < NFDCHUNKS && u->u_ofilep[k] == NULL)
				;
			    if (k >= NFDCHUNKS)
				break;
			    if (kread((KA_T)u->u_ofilep[k],
				      (char *)&u->u_ofile,
				      sizeof(struct ofile_t)))
				break;
			    j = 0;
			}
			j++;
			if (u->u_ofile.ofile[j - 1])
#endif

#if	defined(HPUX8)
		for (i = 0, j = SFDCHUNK; i < p->p_maxof; i++) {
			if (j >= SFDCHUNK) {
			    if (kread((KA_T)p->p_ofilep, (char *)&ofp,
				       sizeof(ofp)))
			    {
				(void) fprintf(stderr,
				    "%s: PID %d, can't read ofilep from %#x\n",
				    Pn, Pid, ofp);
				exit(1);
			    }
			    if (ofp == NULL)
				break;
			    if (kread((KA_T)ofp, (char *)&oft, sizeof(oft))) {
				(void) fprintf(stderr,
				    "%s: PID %d, can't read ofile_t from %#x\n",
				    Pn, Pid, ofp);
				exit(1);
			    }
			    j = 0;
			    p->p_ofilep++;
			}
			j++;
			if (oft.ofile[j - 1])
#endif

#if	defined(_IBMR2)
		for (i = 0; i < u->u_maxofile; i++) {
			if (u->u_ufd[i].fp)
#endif

#if	defined(sgi)
	/*
	 * The SGI file pointers are stored in a dynamically allocated array
	 * that follows the user area segment.  The array is terminated with
	 * a NULL file pointer.
	 */
		for (i = 0; i < nf; i++) {
			if (fp[i] == NULL)
				break;
	
#endif	/* sgi */

#if	defined(NeXT) || defined(sun)
		for (i = 0; i < nf; i++) {
			if (uf[i])
#endif

			{
                                (void) sprintf(Fd, "%4d", i);
				printprep(u->u_comm, NULL, fprint, p->p_uid);

#if	defined(DYNIX)
				printfile((KA_T)uf[i].of_file);
#endif

#if	defined(eta10)
				printfile((KA_T)u->u_ofile[i]);
#endif

#if	defined(HPUX7)
				printfile((KA_T)u->u_ofile.ofile[j - 1]);
#endif

#if	defined(HPUX8)
				printfile((KA_T)oft.ofile[j - 1]);
#endif

#if	defined(_IBMR2)
                                printfile((KA_T)u->u_ufd[i].fp);
#endif

#if	defined(NeXT)
				printfile((KA_T)uf[i]);
#endif

#if	defined(sgi)
				printfile((KA_T)fp[i]);
#endif

#if	defined(sun)
                                printfile((KA_T)uf[i]);
#endif

			}
		}
	}
	for (sfp = Sfile; sfp; sfp = sfp->next) {
		if (sfp->f == 0)
			exit(1);
	}

#if	defined(DYNIX) || defined(_IBMR2) || defined(NeXT)
	exit(0);
#endif

#if	defined(eta10) || defined(hpux) || defined(sgi) || defined(sun)
	return(0);
#endif

}
