/*
 *	mem.c - kernel memory reading functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: mem.c,v 1.2 92/09/05 13:45:40 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


/*
 * kread() - read from kernel memory
 */

int
kread(addr, buf, len)
	KA_T addr;			/* kernel memory address */
	char *buf;			/* buffer to receive data */
	READLEN_T len;			/* length to read */
{
	int br;

#if	defined(DEHINSG)

# if	defined(DYNIX)
	if (Notkmem) {
		if ((addr = vtophys(addr)) == (off_t)-1)
			return(-1);
	}
# endif

#if	defined(DEHIN)
	if (lseek(Kmem, addr, L_SET) == (off_t)-1L)
#endif

#if	defined(sgi)
	if (lseek(Kmem, (off_t)(addr & 0x7fffffff), L_SET) == (off_t)-1L)
#endif

		return(-1);
	br = read(Kmem, buf, len);
#endif

#if	defined(sun)
	br = kvm_read(Kd, (u_long)addr, buf, len);
#endif

	return((br == len) ? 0 : 1);
}


#if	defined(_IBMR2)
/*
 * kreadx() - read kernel segmented memory
 */

int
kreadx(addr, buf, len, sid)
	off_t addr;			/* kernel address */
	char *buf;			/* destination buffer */
	int len;			/* length */
	long sid;			/* segment ID */
{
	int br;

	if (lseek(Mem, addr, L_SET) == (off_t)-1)
		return(1);
	br = readx(Mem, buf, len, sid);
	return (br == len ? 0 : 1);
}
#endif	/* _IBMR2 */


#if	defined(eta10) || defined(HPUX7)
/*
 * mread() -- read from /dev/mem
 */

int
mread(addr, buf, len)
	KA_T addr;			/* /dev/mem address */
	char *buf;			/* buffer to receive data */
	READLEN_T len;			/* length to read */
{
	int br;

	if (lseek(Mem, addr, L_SET) == (off_t)-1L)
		return(1);
	br = read(Mem, buf, len);
	return((br == len) ? 0 : 1);
}
#endif
