/*
 *	misc.c - miscellaneous support functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: misc.c,v 1.4 92/09/06 11:16:33 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


#if	defined(_IBMR2)
/*
 * getchan() - get channel from file path name
 */

int
getchan(p)
	char *p;			/* file path name */
{
	int ch;
	char *s;

	if ((s = strrchr(p, '/')) == NULL) 
		return(-1);
	if (*(++s) == '\0')
		return(-1);
	for (ch = 0; *s; s++) {
		if ( ! isascii(*s) || ! isdigit(*s))
			return(-1);
		ch = (ch * 10) + *s - '0';
	}
	return(ch);
}
#endif


/*
 * isfile() - is file selected for printing
 */

int
isfile(d, ty, ch, i)
	dev_t d;			/* device */

#if	defined(DHINS)
	enum vtype ty;			/* vnode type */
#endif

#if	defined(eta10)
	int ty;				/* type */
#endif

#if	defined(DHNS)
	int ch;				/* dummy */
#endif

#if	defined(_IBMR2)
	chan_t ch;			/* gnode channel */
#endif

	long i;			 	/* inode */

/* ARGSUSED */

{
	int f;
	struct sfile *s;

	for (f = 0, s = Sfile; s; s = s->next) {

#if	defined(_IBMR2)
	/*
	 * Check for a regular AIX multiplexed file.  Match the channel if
	 * it was supplied.
	 */
		if (ty == VMPC) {
			if (s->type && s->mode == S_IFCHR && d == s->dev
			&&  (s->ch < 0 || ch == s->ch)) {
				f = 1;
				break;
			}
			continue;
		}
#endif	/* _IBMR2 */

#if	defined(sgi)
/*
 * Check for an SGI clone file.
 */
		if (Ntype == N_COM) {
			if (major(s->dev) == USEMA_MAJOR
			&&  major(d) == USEMA_MAJOR) {
				f = 1;
				break;
			}
			if (major(s->dev) == CLN2_MAJOR
			&&  major(d) == minor(s->dev)) {
				f = 1;
				break;
			}
		}
#endif	/* sgi */

#if	defined(sun)
	/*
	 * Check for a Sun clone file.
	 */
		if (Ntype == N_STREAM && major(s->dev) == CLONEMAJ
		&& major(d) == minor(s->dev)) {
			f = 1;
			break;
		}
#endif	/* sun */

	/*
	 * Check for a regular file or directory -- the device and
	 * inode numbers must match.
	 */

		if (s->type) {
			if (d == s->dev && (ino_t)i == s->i) {
				f = 1;
				break;
			}
			continue;
		}
	/*
	 * Check for a file system match.
	 * Try to avoid matching character files to non-character devices.
	 */
		if (d == s->dev) {

#if	defined(HASVNODES)
			if ( ! (ty == VCHR && s->mode != S_IFCHR))
#else
			if ( ! (ty == IFCHR && s->mode != S_IFCHR))
#endif

			{
				f = 1;
				break;
			}
		}
	}
/*
 * Convert the name if a match occurred.
 */
	if (f) {
		(void) strcpy(Namech, s->name);

#if	defined(_IBMR2)
		if (ty == VMPC && s->ch < 0)
			(void) sprintf(endnm(), "/%d", ch);
#endif

		if (s->devnm)
			(void) sprintf(endnm(), " (%s)", s->devnm);
		s->f = 1;
		return(1);
	}
	return(0);
}


#if	defined(_IBMR2)
/*
 * isglocked() - is a gnode locked
 */

char
isglocked(ga)
	struct gnode *ga;		/* local gnode address */
{

	struct filock f;
	off_t l;

	if (ga->gn_filocks == NULL)
		return(' ');
	if (kread((KA_T)ga->gn_filocks, &f, sizeof(f)))
		return(' ');
	if (f.set.l_whence == 0 && f.set.l_start == 0 && f.set.l_len == MAXEND)
		l = 1;
	else
		l = 0;
	switch (f.set.l_type & (F_RDLCK | F_WRLCK)) {

	case F_RDLCK:
		return((l) ? 'R' : 'r');
	case F_WRLCK:
		return((l) ? 'W' : 'w');
	case (F_RDLCK + F_WRLCK):
		return('*');
	}
	return(' ');
}
#endif	/* _IBMR2 */


#if	defined(HASSTREAMS)
/*
 * readstdata() - read stream's stdata structure
 */

int
readstdata(addr, buf)
	struct stdata *addr;		/* stdata address in kernel*/
	struct stdata *buf;		/* buffer addess */
{
	if (addr == NULL
	||  kread((KA_T)addr, buf, sizeof(struct stdata))) {
		(void) sprintf(Namech, "no stream data in %#x\n", addr);
		return(1);
	}
	return(0);
}


/*
 * readsthead() - read stream head
 */

int
readsthead(addr, buf)
	struct queue *addr;		/* starting queue pointer in kernel */
	struct queue *buf;		/* buffer for queue head */
{
	struct queue *qp;

	if (addr == NULL) {
		(void) strcpy(Namech, "no stream queue head");
		return(1);
	}
	for (qp = addr; qp; qp = buf->q_next) {
		if (kread((KA_T)qp, (char *)buf, sizeof(struct queue))) {
			(void) sprintf(Namech, "bad stream queue link at %#x",
				qp);
			return(1);
		}
	}
	return(0);
}


/*
 * readstidnm() - read stream module ID name
 */

int
readstidnm(addr, buf, len)
	char *addr;			/* module ID name address in kernel */
	char *buf;			/* receiving buffer address */
	READLEN_T len;			/* buffer length */
{
	if (addr == NULL ||  kread((KA_T)addr, buf, len)) {
		(void) sprintf(Namech, "can't read module ID name from %#x",
			addr);
		return(1);
	}
	return(0);
}


/*
 * readstmin() - read stream's module info
 */

int
readstmin(addr, buf)
	struct module_info *addr;	/* module info address in kernel */
	struct module_info *buf;	/* receiving buffer address */
{
	if (addr == NULL
	||  kread((KA_T)addr, (char *)buf, sizeof(struct module_info))) {
		(void) sprintf(Namech, "can't read module info from %#x", addr);
		return(1);
	}
	return(0);
}


/*
 * readstqinit() - read stream's queue information structure
 */

int
readstqinit(addr, buf)
	struct qinit *addr;		/* queue info address in kernel */
	struct qinit *buf;		/* receiving buffer address */
{
	if (addr == NULL
	||  kread((KA_T)addr, (char *)buf, sizeof(struct qinit))) {
		(void) sprintf(Namech, "can't read queue info from %#x", addr);
		return(1);
	}
	return(0);
}
#endif	/* HASSTREAMS */


/*
 * statsafely() - stat path safely (i. e., with timeout)
 */

int
statsafely(path, buf, tm)
        char *path;
        struct stat *buf;
        int tm;
{
        int err;
        static int sverr;

        if (tm == 0)
                return(stat(path, buf));
        sverr = errno;
        if (setjmp(Tmo_jbuf)) {
                (void) alarm(0);
                (void) signal(SIGALRM, SIG_IGN);
                errno = ETIMEDOUT;
                return(-1);
        }
        (void) signal(SIGALRM, stattimeout);
        (void) alarm((unsigned)tm);
        err = stat(path, buf);
        if (err != 0)
                sverr = errno;
        (void) alarm(0);
        (void) signal(SIGALRM, SIG_IGN);
        errno = sverr;
        return(err);
}


/*
 * stattimeout() - time out a stat() call
 */

#if	defined(DYNIX)
int
#endif

#if	defined(EHINSSG)
void
#endif

stattimeout()
{
        (void) alarm(0);
        (void) signal(SIGALRM, SIG_IGN);
        longjmp(Tmo_jbuf, 1);
}


#if	defined(DYNIX)
/*
 * vtophys() - map kernel virtual address to physical address
 */

off_t
vtophys(vaddr)
	off_t vaddr;
{
	off_t paddr;
	
# if	defined(i386)
	if (vaddr < (off_t)8192)
		return(vaddr);
# endif

	paddr = (off_t)((int *)Sysmap + (vaddr / NBPG));
	if (lseek(Kmem, (off_t)paddr, L_SET) == (off_t)-1)
		return((off_t)-1);
	if (read(Kmem, (char *)&paddr, sizeof(paddr)) != sizeof(paddr))
		return((off_t)-1);
	return ((paddr & ~PTBITS) | (vaddr & PTBITS));
}
#endif	/* DYNIX */
