/*
 *	mnt.c - mount support functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: mnt.c,v 1.7 92/10/09 10:27:33 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


#if	defined(DHNS)
/*
 * completevfs() - complete local vfs structure
 */

# if	defined(DYNIX) || defined(HPUX7) || defined(NeXT) || defined(sun)
void
completevfs(vfs, dev)
	struct l_vfs *vfs;		/* local vfs structure pointer */
	dev_t dev;			/* device */
# endif

# if	defined(HPUX8)
void
completevfs(vfs, dev, v)
	struct l_vfs *vfs;		/* local vfs structure pointer */
	dev_t dev;			/* device */
	struct vfs *v;			/* kernel vfs structure */
# endif

{

#if	defined(HPUX8)
	struct stat sb;
#endif

	struct mounts *mp;

# if	defined(HPUX8)
/*
 * On HP-UX 8.x search first for a match on the file system name
 * from the vfs structure.
 */
	if (v) {
		for (mp = Mtab; mp; mp = mp->next) {
			if (strcmp(mp->fsname, v->vfs_name) == 0) {
				vfs->dev = mp->dev;
				vfs->dir = mp->dir;
				vfs->fsname = mp->fsname;
				return;
			}
		}
	}
# endif

/*
 * Search for a match on device number.
 */
	for (mp = Mtab; mp; mp = mp->next) {
		if (mp->dev == dev) {

# if	defined(DYNIX) || defined(hpux)
			vfs->dev = mp->dev;
# endif

			vfs->dir = mp->dir;
			vfs->fsname = mp->fsname;
			return;
		}
	}

# if	defined(HPUX8)
/*
 * If the file system name and device number searches fail, use the
 * vfs structure name, if there is one.
 */
	if (v && v->vfs_name) {

	/*
	 * Stat the vfs_name and use its device number to search the mount
	 * table once more.
	 */
		if (statsafely(v->vfs_name, &sb, STATTMO) == 0) {
			for (mp = Mtab; mp; mp = mp->next) {
				if (mp->dev == sb.st_dev) {
					vfs->dev = mp->dev;
					vfs->dir = mp->dir;
					vfs->fsname = mp->fsname;
					return;
				}
			}
		}
	/*
	 * As a last recourse, use the vfs_name.
	 */
		if ((vfs->dir = (char *)malloc(strlen(v->vfs_name)+1))
		== NULL) {
			(void) fprintf(stderr, "%s: no space for vfs name\n",
				Pn);
			exit(1);
		}
		(void) strcpy(vfs->dir, v->vfs_name);
		vfs->dev = mp->dev;
	}
# endif	/* HPUX8 */

}
#endif	/* DYNIX || hpux || NeXT || sun */


#if	defined(eta10) || defined(sgi)
/*
 * readfsinfo() - read file system information
 */

int
readfsinfo()
{
	char *cp;
	int i;
	MALLOC_S len;

/*
 * Access the file system information table in the kernel.
 */
	if (Nl[X_NFSTYP].n_value == NULL
	||  kread((KA_T)Nl[X_NFSTYP].n_value, (char *)&Nfstyp, sizeof(Nfstyp))
	||  Nfstyp < 1) {
		(void) fprintf(stderr,
			"%s: nfstyp value (%d) error, kernel = %#x\n",
			Pn, Nfstyp, Nl[X_NFSTYP].n_value);
		return(0);
	}
	len = (MALLOC_S)(Nfstyp * sizeof(struct fsinfo));
	if (Nl[X_FSINFO].n_value == NULL
	||  (Fsinfo = (struct fsinfo *)malloc(len)) == NULL
	||  kread((KA_T)Nl[X_FSINFO].n_value, (char *)Fsinfo, len)) {
		(void) fprintf(stderr,
			"%s: fstyp read error, length = %d, kernel = %#x\n",
			Pn, len, Nl[X_FSINFO].n_value);
		return(0);
	}
/*
 * Get the file system names.
 */
	for (i = 0; i < Nfstyp; i++) {
	    if (Fsinfo[i].fs_name) {
		if ((cp = malloc(FSNAMEL)) == NULL) {
		    (void) fprintf(stderr,
			"%s: no space for file system name[%d]\n", Pn, i);
				return(0);
		}
		if (kread((KA_T)Fsinfo[i].fs_name, cp, FSNAMEL)) {
		    (void) fprintf(stderr,
			"%s: can't read file system name[i] from %#x\n",
			Pn, i, Fsinfo[i].fs_name);
		}
		cp[FSNAMEL - 1] = '\0';
		Fsinfo[i].fs_name = cp;
	    }
	}
	return(1);
}
#endif	/* eta10 || sgi */


#if	defined(DHNS) || defined(sgi)
/*
 * readmnt() - read mount table
 */

int
readmnt()
{
	int err = 0;
	FILE *mfp;
	struct mntent *mp;
	struct mounts *mtp;
	struct stat sb;

        if ((mfp = setmntent(MOUNTED, "r")) == NULL) {
                (void) fprintf(stderr, "%s: can't access %s\n", Pn, MOUNTED);
                return(0);
        }
        while ((mp = getmntent(mfp)) != NULL) {

# if	defined(sun)
	/*
	 * Skip SunOS loop-back mounts, since they are aliases for legitimate
	 * file systems and there is no way to determine that a vnode refers
	 * to a loop-back alias.
	 */
		if (strcmp(mp->mnt_type, MNTTYPE_LO) == 0)
			continue;
# endif

                if (statsafely(mp->mnt_dir, &sb, STATTMO))
                        continue;
		if ((mtp = (struct mounts *)malloc(sizeof(struct mounts)))
		== NULL) {
			err = 1;
			break;
		}
		if ((mtp->dir = (char *)malloc(
				(MALLOC_S)(strlen(mp->mnt_dir)+1)))
		== NULL) {
			err = 1;
			break;
		}
		(void) strcpy(mtp->dir, mp->mnt_dir);
		if ((mtp->fsname = (char *)malloc(
				   (MALLOC_S)(strlen(mp->mnt_fsname)+1)))
		== NULL) {
			err = 1;
			break;
		}
		(void) strcpy(mtp->fsname, mp->mnt_fsname);
		mtp->next = Mtab;
		mtp->dev = sb.st_dev;
		mtp->inode = sb.st_ino;
		mtp->mode = sb.st_mode;
		mtp->rdev = sb.st_rdev;
		Mtab = mtp;
        }
        (void) endmntent(mfp);
	if (err) {
		(void) fprintf(stderr, "%s: no space for mount at %s (%s)\n",
			Pn, mp->mnt_fsname, mp->mnt_dir);
		return(0);
	}
	return(1);
}
#endif	/* DYNIX || hpux || NeXT || sun */


#if	defined(eta10)
/*
 * readmnt() - read mount table
 */

int
readmnt()
{
	int fd, rv = 1;
	struct mnttab m;
	struct mounts *mtp;
	struct stat sb;

/*
 * Open and read the mount table.
 */
	if ((fd = open(MNTTAB, O_RDONLY, 0)) < 0) {
		(void) fprintf(stderr,
			"%s: open %s: %s\n", Pn, MNTTAB, sys_errlist[errno]);
		return(0);
	}
	while (read(fd, (char *)&m, sizeof(m)) == sizeof(m)) {
		if (statsafely(m.mt_filsys, &sb, STATTMO))
			continue;
		if ((mtp = (struct mounts *)malloc(sizeof(struct mounts)))
		== NULL) {
			(void) fprintf(stderr,
				"%s: no space for internal mount table\n", Pn);
			rv = 0;
			break;
		}
                if ((mtp->dir = (char *)malloc(
                                (MALLOC_S)(strlen(m.mt_filsys)+1)))
                == NULL) {
			(void) fprintf(stderr,
			    "%s: no space for internal mount directory: %s\n",
			    Pn, m.mt_filsys);
                        rv = 0;
                        break;
                }
                (void) strcpy(mtp->dir, m.mt_filsys);
                if ((mtp->fsname = (char *)malloc(
                                   (MALLOC_S)(strlen(m.mt_dev)+1)))
                == NULL) {
			(void) fprintf(stderr,
			    "%s: no space for internal mount file system: %s\n",
			    Pn, m.mt_dev);
                        rv = 0;
                        break;
                }
                (void) strcpy(mtp->fsname, m.mt_dev);
                mtp->next = Mtab;
                mtp->dev = sb.st_dev;
                mtp->inode = sb.st_ino;
                mtp->mode = sb.st_mode;
                mtp->rdev = sb.st_rdev;
                Mtab = mtp;
	}
	(void) close(fd);
	return(rv);
}
#endif	/* eta10 */


#if	defined(_IBMR2)
/*
 * readmnt() - read mount table
 */

int
readmnt()
{
	int err = 0;
	char *dir, *fs, *h;
	int fsl;
	struct mounts *mtp;
	int nm;
	struct stat sb;
	unsigned sz;
	struct vmount *v, *vt;

/*
 * Read the table of vmount structures.
 */
	for (sz = sizeof(struct vmount);;) {
		if ((vt = (struct vmount *)malloc(sz)) == NULL) {
			(void) fprintf(stderr,
				"%s: no space for vmount table\n", Pn);
			return(0);
		}
		nm = mntctl(MCTL_QUERY, sz, vt);
		if (nm > 0) {
			if (vt->vmt_revision != VMT_REVISION) {
				(void) fprintf(stderr,
					"%s: stale file system, rev %d != %d\n",
					Pn, vt->vmt_revision, VMT_REVISION);
				return(0);
			}
			break;
		}
		if (nm == 0) {
			sz = (unsigned)vt->vmt_revision;
			(void) free((FREE_P *)vt);
		} else {
			(void) fprintf(stderr, "%s: mntctl error: %s\n",
				Pn, sys_errlist[errno]);
			return(0);
		}
	}
/*
 * Scan the vmount structures and build Mtab.
 */
	for (v = vt; nm--; v = (struct vmount *)((char *)v + v->vmt_length)) {
		dir = (char *)vmt2dataptr(v, VMT_STUB);
		fs = (char *)vmt2dataptr(v, VMT_OBJECT);
		h = (char *)vmt2dataptr(v, VMT_HOST);
		fsl = strlen(fs);
		if (v->vmt_flags & MNT_REMOTE)
			fsl += strlen(h) + 1;
                if (statsafely(dir, &sb, STATTMO))
                        continue;
		if ((mtp = (struct mounts *)malloc(sizeof(struct mounts)))
		== NULL) {
			err = 1;
			break;
		}
		if ((mtp->dir = (char *)malloc(strlen(dir) + 1)) == NULL) {
			err = 1;
			break;
		}
		(void) strcpy(mtp->dir, dir);
		if ((mtp->fsname = (char *)malloc(fsl + 1)) == NULL) {
			err = 1;
			break;
		}
		if (v->vmt_flags & MNT_REMOTE)
			(void) sprintf(mtp->fsname, "%s:%s", h, fs);
		else
			(void) strcpy(mtp->fsname, fs);
		mtp->dev = sb.st_dev;
		mtp->inode = sb.st_ino;
		mtp->mode = sb.st_mode;
		mtp->rdev = sb.st_rdev;
		mtp->next = Mtab;
		Mtab = mtp;
        }
	if (err) {
		(void) fprintf(stderr, "%s: no space for mount at %s (%s)\n",
			Pn, fs, dir);
		return(0);
	}
	(void) free((FREE_P *)vt);
	return(1);
}
#endif	/* _IBMR2 */


#if	defined(HASVNODES)
/*
 * readvfs() - read vfs structure
 */

struct l_vfs *
readvfs(va)
	struct vfs *va;			/* vfs structure kernel address
					 * - e. g., v.v_vfsp */
{
	struct vfs v;
	struct l_vfs *vp;

# if	defined(DYNIX) ||defined(hpux) || defined(sun)
	struct mount m;
# endif

# if	defined(DYNIX) || defined(hpux)
	struct mntinfo mi;
# endif

# if	defined(_IBMR2)
	struct gfs g;
	void *mp;
	char *s1, *s2;
	unsigned slen;
	u_long ul;
	struct vmount *vm;
# endif

	for (vp = Lvfs; vp; vp = vp->next) {
		if (va == vp->addr)
			return(vp);
	}
	if ((vp = (struct l_vfs *)malloc(sizeof(struct l_vfs))) == NULL) {
		(void) fprintf(stderr, "%s: PID %d, no space for vfs\n",
			Pn, Pid);
		exit(1);
	}

# if	defined(DYNIX) || defined(hpux)
	vp->dev = 0;
# endif

	vp->dir = NULL;
	vp->fsname = NULL;

# if	defined(sun)
/*
 * SunOS vnodes of type VCHR that have a non-null v_stream pointer
 * represent streams and have no virtual file system pointer.
 */

# endif

# if	defined(_IBMR2) && _AIXV>=30200
/*
 * The vnodes of AIX 3.2 FIFO's don't point at a virtual file system
 * structure.
 */

# endif

# if	defined(DYNIX) ||defined(hpux) || (defined(_IBMR2) && _AIXV<30200) || defined(NeXT)

	if (!va)
		goto vfs_exit;
# endif

	if (va && kread((KA_T)va, (char *)&v, sizeof(v))) {

# if	defined(DHIN)

vfs_exit:

# endif

		(void) free((FREE_P *)vp);
		return(NULL);
	}

# if	defined(_IBMR2)
/*
 * Locate AIX mount information.
 */

#  if	_AIXV>=30200
    if (va == NULL) {
	(void) bzero(vp, sizeof(struct l_vfs));
    } else {
#  endif

	if (!v.vfs_gfs || kread((KA_T) v.vfs_gfs, &g, sizeof(g)))
		goto vfs_exit;
	if (!v.vfs_mdata
	||  kread((KA_T) v.vfs_mdata + sizeof(u_long), &ul, sizeof(ul)))
		goto vfs_exit;
	if ((mp = malloc((size_t) ul)) == NULL) {
		(void) fprintf(stderr, "%s: PID %d, no space for mount data\n",
			Pn, Pid);
		exit(1);
	}
	if (kread((KA_T) v.vfs_mdata, mp, (int) ul)) {
		(void) free((FREE_P *)mp);
		goto vfs_exit;
	}
	vm = (struct vmount *)mp;
	vp->vmt_flags = vm->vmt_flags;
	vp->vmt_gfstype = vm->vmt_gfstype;

#  if	_AIXV>=30200
	if (vp->vmt_flags & MNT_REMOTE)
		vp->dev = 0x80000000 | vm->vmt_vfsnumber;
	else
#  endif

		vp->dev = (dev_t)vm->vmt_fsid.fsid_dev;
	if ((s1 = vmt2dataptr(vm, VMT_STUB)) != NULL) {
		if ((vp->dir = (char *)malloc(strlen(s1) + 1)) == NULL) {

readvfs_aix1:
			(void) fprintf(stderr,
				"%s: PID %d, readvfs, no space\n", Pn, Pid);
			exit(1);
		}
		(void) strcpy(vp->dir, s1);
	} else
		vp->dir = NULL;
	s1 = vmt2dataptr(vm, VMT_HOST);
	if ((s2 = vmt2dataptr(vm, VMT_OBJECT)) == NULL || *s1 == '\0')
		s2 = g.gfs_name;
	if (s1 == NULL && s2 == NULL)
		vp->fsname = NULL;
	else {
		slen = s2 ? strlen(s2) : 0;
		if (vm->vmt_flags & MNT_REMOTE)
			slen += (s1 ? strlen(s1) : 0) + 1;
		if ((vp->fsname = (char *)malloc(slen + 1)) == NULL)
			goto readvfs_aix1;
		if (vm->vmt_flags & MNT_REMOTE)
			(void) sprintf(vp->fsname, "%s:%s",
				s1 ? s1 : "", s2 ? s2 : "");
		else
			(void) strcpy(vp->fsname, s2 ? s2 : "");
	}
	(void) free((FREE_P *)mp);

#  if	_AIXV>=30200
    }
#  endif
# endif	/* _IBMR2 */

/*
 * Complete DYNIX, HP-UX and SunOS mount information.
 */

# if	defined(DYNIX) || defined(hpux)
	if (Ntype == N_NFS) {

	/*
	 * DYNIX and HP-UX NFS vnode device values have a major number of 255.
	 * The minor number is a serial number found in the mntinfo
	 * structure to which the virtual file system's vfs_data points.
	 */ 
		if (v.vfs_data
		&&  kread((KA_T)v.vfs_data, (char *)&mi, sizeof(mi)) == 0)

#  if	defined(DYNIX) || defined(HPUX7)
			(void) completevfs(vp, makedev(255, mi.mi_mntno));
#  endif

#  if	defined(HPUX8)
			(void) completevfs(vp, makedev(255, mi.mi_mntno), NULL);
#  endif

	} else {
		if (kread((KA_T)v.vfs_data, (char *)&m, sizeof(m)) == 0)

#  if	defined(DYNIX) || defined(HPUX7)
			(void) completevfs(vp, m.m_dev);
#  endif

#  if	defined(HPUX8)
			(void) completevfs(vp, m.m_dev, &v);
#  endif

	}
# endif

# if	defined(sun)
	if (va && v.vfs_data && kread((KA_T)v.vfs_data, (char *)&m, sizeof(m))
	== 0)
		(void) completevfs(vp, m.m_dev);
# endif

	vp->next = Lvfs;
	vp->addr = va;
	Lvfs = vp;
	return(vp);
}
#endif	/* HASVNODES */


#if	!defined(HASVNODES)
/*
 * srchmounts() - search mounts
 */

struct mounts *
srchmounts(d)
	int d;				/* device to locate */
{
	struct mounts *m;

	for (m = Mtab; m; m = m->next) {
		if (d == m->dev)
			return(m);
	}
	return(NULL);
}
#endif	/* ! HASVNODES */
