/*
 *	node.c - node reading functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: node.c,v 1.9 92/10/09 10:27:36 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


#if	defined(HASVNODES)
/*
 * printnode() - print primary node on a system with vnodes
 */

void
printnode(va)
	caddr_t va;			/* vnode kernel space address */
{

	dev_t dev;
	struct inode i;
	int prdev = 1;
	struct rnode r;
	char *ty;
	enum vtype type;
	struct vnode v;
	struct l_vfs *vfs;

# if	defined(NeXT)
	struct vnode rv;
	struct snode s;
# endif

# if	defined(_IBMR2)
	struct cdrnode c;
	struct gnode g;

#  if	_AIXV>=30200
	struct devnode dn;
	struct gnode pg;
	struct specnode sn;
	struct fifonode f;
#  endif
# endif

# if	defined(sun)
	struct fifonode f;
	int fm;
	struct module_info mi;
	char mn[STRNML];
	struct qinit qi;
	struct vnode rv;
	struct snode s;
	struct stdata sd;
	struct queue sh;
	struct tmpnode t;
	char *tn;
# endif

/*
 * Read the vnode.
 */
	if ( ! va) {
		(void) strcpy(Namech, "no vnode address");
		printline();
		return;
	}
	if (readvnode((caddr_t)va, &v)) {
                printline();
                return;
        }


# if	defined(DHNS)
/*
 * Determine the DYNIX, HP-UX, NeXT and SunOS vnode type.
 */
	if (Nl[X_NFS_OPS].n_value
	&& (unsigned long)v.v_op == Nl[X_NFS_OPS].n_value)
		Ntype = N_NFS;

#  if	defined(sun)
	else if (Nl[X_TMP_OPS].n_value
	     && (unsigned long)v.v_op == Nl[X_TMP_OPS].n_value)
		Ntype = N_TMP;
#  endif

	else if (v.v_type == VFIFO)
		Ntype = N_FIFO;

#  if	defined(sun)
	else if (v.v_type == VCHR && !v.v_vfsp && v.v_stream)
		Ntype = N_STREAM;
#  endif

/*
 * Determine the DYNIX, HP-UX, NeXT or SunOS lock type.
 */
	if (v.v_shlockc || v.v_exlockc) {
		if (v.v_shlockc && v.v_exlockc)
			Lock = '*';
		else if (v.v_shlockc)
			Lock = 'R';
		else
			Lock = 'W';
	}
/*
 * Read the virtual file system structure.
 */
	if ((vfs = readvfs(v.v_vfsp)) == NULL) {
                (void) sprintf(Namech, "can't read vfs for %#x at %#x", va,
			v.v_vfsp);
                printline();
		return;
	}
# endif	/* DYNIX || hpux || NeXT || sun */

# if	defined(_IBMR2)

/*
 * Read the gnode.
 */
	if (!v.v_gnode || readgnode(v.v_gnode, &g)) {
		(void) sprintf(Namech, "vnode at %#x has no gnode\n", va);
		printline();
		return;
	}

#  if	_AIXV>=30200

/*
 * Under AIX 3.2, if the vnode type is ISVDEV, then there is a special
 * node and a fifonode or devnode.  Behind them are the "real" gnode,
 * inode and vnode.
 */
	if (ISVDEV(g.gn_type)) {
		switch (g.gn_type) {
		case VBLK:
			Ntype = N_BLK;
			break;
		case VCHR:
			Ntype = N_CHR;
			break;
		case VFIFO:
			Ntype = N_FIFO;
			break;
		case VMPC:
			Ntype = N_MPC;
			break;
		default:
			(void) sprintf(Namech,
				"vnode at %#x: unknown ISVDEV(%#x)",
				va, g.gn_type);
			printline();
			return;
		}
	/*
	 * Read the special node.
	 */
		if (!g.gn_data
		|| kread((KA_T)g.gn_data, (char *)&sn, sizeof(sn))
		!= 0) {
			(void) sprintf(Namech,
				"vnode at %#x: can't read specnode (%#x)",
				va, g.gn_data);
			printline();
		}
	/*
	 * Read the PFS gnode and its inode and vnode.
	 */
		if (sn.sn_pfsgnode) {
			if (readgnode(sn.sn_pfsgnode, &g)) {
				(void) sprintf(Namech,
				    "vnode at %#x: can't read pfsgnode (%#x)",
				    va, sn.sn_pfsgnode);
				printline();
				return;
			}
			if (!g.gn_data || readinode(g.gn_data, &i)) {
				(void) sprintf(Namech,
				    "pfsgnode at %#x: can't read inode (%#x)",
				    sn.sn_pfsgnode, g.gn_data);
				printline();
				return;
			}
			if (!g.gn_vnode || readvnode((caddr_t)g.gn_vnode, &v)) {
				(void) sprintf(Namech,
				    "pfsgnode at %#x: can't read vnode (%#x)",
				    sn.sn_pfsgnode, g.gn_vnode);
				printline();
				return;
			}
		} else {
			(void) bzero((char *)&i, sizeof(i));
			i.i_number = -2l;
		}
	/*
	 * If it's a FIFO, read its fifonode.
	 */
		if (Ntype == N_FIFO) {
			if (!sn.sn_fifonode
			|| readfifonode(sn.sn_fifonode, &f)) {
				(void) sprintf(Namech,
				    "vnode at %#x: can't read fifonode (%#x)",
				    va, sn.sn_fifonode);
				printline();
				return;
			}
	/*
	 * Otherwise, read the devnode and its gnode.
	 */
		} else {
			if (!sn.sn_devnode
			|| kread((KA_T)sn.sn_devnode,(char *)&dn,sizeof(dn))) {
				(void) sprintf(Namech,
				    "vnode at %#x: can't read devnode (%#x)",
				    va, sn.sn_devnode);
				printline();
				return;
			}
			g = dn.dv_gnode;
		}
	}
#  endif	/* _AIXV>=30200 */

/*
 * Read the AIX virtual file system structure.
 */

	if (g.gn_rdev == NODEVICE) {
		vfs = NULL;
		(void) sprintf(Devch, "0x%08x", va);
	} else {
		if ((vfs = readvfs(v.v_vfsp)) == NULL) {
			(void) sprintf(Namech,
				"can't read vfs for %#x at %#x",
				va, v.v_vfsp);
			printline();
			return;
		}
	}
/*
 * Set the type for an NFS vnode.
 * Get the lock status.
 */
	if (vfs && vfs->vmt_flags & MNT_REMOTE)
		Ntype = N_NFS;
	Lock = isglocked(&g);
	switch (Ntype) {

/*
 * Read an AIX, NFS rnode.
 */
	case N_NFS:
		if (g.gn_data == NULL || readrnode(g.gn_data, &r)) {
			(void) sprintf(Namech,
				"remote gnode at %#x has no rnode", v.v_gnode);
			printline();
			return;
		}
		break;
/*
 * Read N_REG nodes.
 */
	case N_REG:
		if (vfs && vfs->vmt_gfstype == MNT_CDROM) {

		/*
		 * Read the AIX, CDROM cdrnode.
		 */
			if (g.gn_data == NULL || readcdrnode(g.gn_data, &c)) {
				(void) sprintf(Namech,
					"gnode at %#x has no cdrnode",
					v.v_gnode);
				printline();
				return;
			}
			(void) bzero((char *)&i, sizeof(i));
			i.i_number = c.cn_inumber;
			i.i_size = (off_t)c.cn_size;
		/*
		 * Otherwise, read the inode.
		 */

		} else if (!g.gn_data || readinode(g.gn_data, &i)) {
			(void) sprintf(Namech, "gnode at %#x has no inode",
				v.v_gnode);
			printline();
			return;
		}
	}
# endif	/* _IBMR2 */

# if	defined(DHNS)
/*
 * Read the DYNIX, HP-UX, NeXT or SunOS fifonode, inode, rnode, snode or
 * tmpnode.
 */
	switch (Ntype) {
	case N_NFS:
		if (!v.v_data || readrnode((caddr_t)v.v_data, &r)) {
			(void) sprintf(Namech,
				"vnode at %#x: can't read rnode (%#x)",
				va, v.v_data);
			printline();
			return;
		}
		break;

#  if	defined(sun)
	case N_FIFO:

	/*
	 * A Sun FIFO's vnode is linked to a fifonode.  The s_realvp
	 * pointer of the snode contained in the fifonode points to
	 * a vnode that points to the inode.  (Whew!)
	 */
		if (!v.v_data || readfifonode(v.v_data, &f)) {
			(void) sprintf(Namech,
				"vnode at %#x: can't read fifonode (%#x)",
				va, v.v_data);
			printline();
			return;
		}
		if (f.fn_snode.s_realvp) {
			if (readvnode((caddr_t)f.fn_snode.s_realvp, &rv)) {
			    (void) sprintf(Namech,
				"fifonode at %#x: can't read real vnode (%#x)",
				v.v_data, f.fn_snode.s_realvp);
			    printline();
			    return;
			}
			if (!rv.v_data || readinode(rv.v_data, &i)) {
				(void) sprintf(Namech,
				    "fifonode at %#x: can't read inode (%#x)",
				    v.v_data, rv.v_data);
				printline();
				return;
			}
		} else {
			printline();
			return;
		}
		break;
	case N_STREAM:
		Inode = -2L;
		(void) strcpy(Iproto, "STR");
		fm = 0;
		if (!v.v_data || readsnode((caddr_t)v.v_data, &s)) {
			(void) sprintf(Namech,
				"vnode at %#x: can't read snode (%#x)",
				va, v.v_data);
			printline();
			return;
		}
		if (v.v_stream) {

		/*
		 * Follow the stream structure from its stdata pointer to the
		 * module information structure.  Then read the first STRNML-1
		 * characters of the module name.
		 */
			if (readstdata(v.v_stream, &sd) == 0
			&&  readsthead(sd.sd_wrq, &sh) == 0
			&&  readstqinit(sh.q_qinfo, &qi) == 0
			&&  readstmin(qi.qi_minfo,&mi) == 0
			&&  readstidnm(mi.mi_idname, mn, STRNML-1) == 0) {
				mn[STRNML-1] = '\0';
				fm = 1;
			}
		}
		break;
	case N_TMP:
		if (!v.v_data || readtnode((caddr_t)v.v_data, &t)) {
			(void) sprintf(Namech,
				"vnode at %#x: can't read tnode (%#x)",
				va, v.v_data);
			printline();
			return;
		}
		break;
#  endif	/* sun */

	case N_REG:
	default:

#  if	defined(NeXT) || defined(sun)
	/*
	 * NeXT and Sun VCHR vnodes point to an snode.  The snode's s_realvp
	 * usually points to a real vnode, which points to an inode.
	 */
		if (v.v_type == VCHR) {
			if (!v.v_data || readsnode(v.v_data, &s)) {
				(void) sprintf(Namech,
					"vnode at %#x: can't read snode(%#x)",
					va, v.v_data);
				printline();
				return;
			}
			if (s.s_realvp) {
			    if (readvnode((caddr_t)s.s_realvp, &rv)) {
				(void) sprintf(Namech,
				    "snode at %#x: can't read real vnode (%#x)",
				    v.v_data, s.s_realvp);
				printline();
				return;
			    }
			    if (!rv.v_data || readinode(rv.v_data, &i)) {
				(void) sprintf(Namech,
					"snode at %#x: can't read inode (%#x)",
					v.v_data, rv.v_data);
				printline();
				return;
			    }
			} else
				(void) bzero((char *)&i, sizeof(i));
			break;
		}
#  endif	/* NeXT || sun */

		if (!v.v_data || readinode(v.v_data, &i)) {
			(void) sprintf(Namech,
				"vnode at %#x: can't read inode (%#x)",
				va, v.v_data);
			printline();
			return;
		}
	}
# endif	/* DYNIX || hpux || NeXT || sun */

/*
 * Get device and type for printing.
 */

# if	defined(DYNIX)
	switch(Ntype) {
	case N_NFS:

	/*
	 * Neither the rnode nor the vnode under DYNIX 3.0.12 contains the
	 * correct device number, so it's reclaimed (if possible) from the
	 * local, virtual file system table entry (put there by completevfs()
	 * from information placed in Mtab by readmnt()).
	 */
		dev = vfs->dev;
		break;
	default:
		dev = (v.v_type == VCHR) ? v.v_rdev : i.i_dev;
	}
	type = v.v_type;
	if (vfs->dir == NULL)
		(void) completevfs(vfs, dev);
# endif

# if	defined(hpux)
	if (Ntype == N_NFS)
		dev = vfs->dev;
	else

#  if	defined(HPUX7)
		dev = (v.v_type == VCHR) ? v.v_rdev : i.i_dev;
#  endif

#  if	defined(HPUX8)
		if (v.v_type == VCHR)
			dev = v.v_rdev;
		else if (vfs->fsname)
			dev = vfs->dev;
		else
			dev = i.i_dev;
#  endif

	type = v.v_type;
# endif

# if	defined(_IBMR2)
	if (Ntype == N_NFS)
		dev = vfs->dev;
	else
		dev = g.gn_rdev;

#  if	_AIXV>=30200
	if (Ntype == N_MPC)
		type = VMPC;
	else
#  endif
		type = g.gn_type;
# endif

# if	defined(NeXT)
	switch (Ntype) {
	case N_NFS:
		dev = r.r_attr.va_fsid;
		if (dev & 0x8000)
			dev |= 0xff00;
		break;
	case N_REG:
		dev = (v.v_type == VCHR) ? v.v_rdev : i.i_dev;
	}
	type = v.v_type;
	if (vfs->dir == NULL)
		(void) completevfs(vfs, dev);
# endif

# if	defined(sun)
	switch (Ntype) {
	case N_NFS:
		dev = r.r_attr.va_fsid;
		break;
	case N_STREAM:
		dev = s.s_dev;
		break;
	case N_TMP:
		dev = t.tn_attr.va_fsid;
		break;
	default:
		dev = (v.v_type == VCHR) ? v.v_rdev : i.i_dev;
	}
	type = v.v_type;
	if (vfs->dir == NULL)
		(void) completevfs(vfs, dev);
# endif

/*
 * Obtain the inode number.
 */

# if	defined(DYNIX)
	switch (Ntype) {
	case N_NFS:
		Inode = (long)r.r_nfsattr.na_nodeid;
		break;
	case N_FIFO:
	case N_REG:
		Inode = (long)i.i_number;
	}
# endif

# if	defined(hpux)
	switch (Ntype) {
	case N_NFS:
		Inode = (long)r.r_nfsattr.na_nodeid;
		break;
	case N_FIFO:
	case N_REG:
		Inode = (long)i.i_number;
	}
# endif

# if	defined(_IBMR2)
	switch (Ntype) {

	case N_NFS:
		Inode = (long) r.r_attr.va_serialno;
		break;

#  if	_AIXV>=30200
	case N_FIFO:
	case N_BLK:
	case N_CHR:
	case N_MPC:
#  endif

	case N_REG:
		Inode = (long) i.i_number;
	}
# endif

# if	defined(NeXT)
	switch(Ntype) {
	case N_NFS:
		Inode = (long)r.r_attr.va_nodeid;
		break;
	case N_REG:
		Inode = (long)i.i_number;
	}
# endif

# if	defined(sun)
	switch (Ntype) {
	case N_NFS:
		Inode = (long)r.r_attr.va_nodeid;
		break;
	case N_FIFO:
	case N_REG:
		Inode = (long)i.i_number;
		break;
	case N_TMP:
		Inode = (long)t.tn_attr.va_nodeid;
		break;
	}
# endif

/*
 * Obtain the file size.
 */
       if( ! Foffset) {
	    switch (Ntype) {

# if	defined(_IBMR2) && _AIXV>=30200
	    case N_FIFO:
		Size = f.ff_size;
		Size_def = 1;
		break;
# endif

	    case N_NFS:

# if	defined(DYNIX) || defined(hpux)
		Size = r.r_nfsattr.na_size;
# endif

# if	defined(_IBMR2) || defined(NeXT) || defined(sun)
		Size = r.r_attr.va_size;
# endif

		Size_def = 1;
		break;

# if	defined(_IBMR2) && _AIXV>=30200
	    case N_BLK:
		Size = i.i_size;
		Size_def = 1;
		break;

	    case N_CHR:
	    case N_MPC:
		Offset_def = 1;
		break;
# endif

	    case N_REG:
		if (type == VREG || type == VDIR) {
			Size = i.i_size;
			Size_def = 1;
		}

# if	defined(DHNS)
		else if (type == VCHR)
			Offset_def = 1;
# endif

# if	defined(_IBMR2)
		else if (type == VCHR || type == VMPC)
			Offset_def = 1;
# endif

		break;

# if	defined(sun)
	    case N_STREAM:
		Offset_def = 1;
		break;
	    case N_TMP:
		Size = t.tn_attr.va_size;
		Size_def = 1;
# endif
	    }
	} else
	    Offset_def = 1;

/*
 * Format the name characters.
 */
	if (Ntype == N_NFS && Fnfs)
		Fprint = 1;

#if	defined(sun)
	if (Ntype != N_STREAM) {
#endif

	    if (vfs)
		(void) sprintf(Namech, "%s %s%s%s",
			(vfs->dir)    ? vfs->dir    : "",
			(vfs->fsname) ? "("         : "",
			(vfs->fsname) ? vfs->fsname : "",
			(vfs->fsname) ? ")"         : "");

#if	defined(sun)
	}
#endif

/*
 * Test for specified file.
 */
	if (Sfile && Fprint == 0) {

# if	defined(DHNS)
		Fprint = isfile(dev, type, 0, Inode);
# endif

# if	defined(_IBMR2)
		Fprint = isfile(dev, type, g.gn_chan, Inode);
# endif

		if (Fprint == 0)
			return;
		prdev = 0;
	}
/*
 * Format the vnode type, and possibly the device name.
 */
	switch (type) {

	case VNON:
		ty ="VNON";
		break;
	case VREG:
	case VDIR:
		ty = (type == VREG) ? "VREG" : "VDIR";
		Fdev = 1;
		Dev = dev;
		break;
	case VBLK:
		ty = "VBLK";
		break;
	case VCHR:
		Dev = dev;
		Fdev = 1;
		if (prdev)
			(void) printchdevname(Dev, -1);
		ty = "VCHR";

#if	defined(sun)
		if (Ntype == N_STREAM && fm)
			(void) sprintf(endnm(), "->%s", mn);
#endif

		break;
	case VLNK:
		ty = "VLNK";
		break;
	case VSOCK:
		ty = "SOCK";
		break;
	case VBAD:
		ty = "VBAD";
		break;
	case VFIFO:
		if (Devch[0] == '\0') {
			Dev = dev;
			Fdev = 1;
		}
		ty = "FIFO";
		break;

# if	defined(_IBMR2)
	case VMPC:
		Dev = g.gn_rdev;

#  if	_AIXV<30200
		Inode = -2l;
#  endif

		Fdev = 1;
		if (prdev)
			(void) printchdevname(g.gn_rdev, g.gn_chan);
		ty = "VMPC";
		break;
# endif

	default:
		(void) sprintf(Type, "%4d", type);
		(void) strcpy(Namech, "unknown type");
		ty = NULL;
	}
	if (ty)
		(void) strcpy(Type, ty);
/*
 * Print the line (at last) if printing has been selected.
 */
	printline();
}
#endif	/* HASVNODES */


#if	!defined(HASVNODES)
/*
 * printnode() - print primary node on a vnode-less system
 */

void
printnode(ia)
	struct inode *ia;		/* inode address in kernel */
{
	struct filock fl;
	struct inode i;
	short ity;
	struct mounts *lm;
	struct mount m;
	struct module_info mi;
	int prdev;
	struct qinit qi;
	struct rnode r;
	struct stdata sd;
	struct queue sh;
	char *tn;
	int type;

/*
 * Read the inode.
 */
	if ( ! ia) {
		(void) strcpy(Namech, "no inode address");
		printline();
		return;
	}
	if (readinode(ia, &i)) {
		printline();
		return;
	}

/*
 * Identify the node type.
 */
	ity = i.i_fstyp;
	type = i.i_ftype & IFMT;
	if (ity < 1 || ity >= Nfstyp || Fsinfo[ity].fs_name == NULL) {
		(void) sprintf(Namech, "unknown fstyp (%d) in inode", ity);
		printline();
		return;
	}
	if (strcasecmp(Fsinfo[ity].fs_name, "NFS") == 0) {

	/*
	 * Get information on NFS file.
	 */
		Ntype = N_NFS;
		if (Fnfs)
			Fprint = 1;
		if (kread((KA_T)i.i_mntdev, (char *)&m, sizeof(m))) {
			(void) sprintf(Namech,
			    "can't read NFS mount structure (%#x)",
			    i.i_mntdev);
			printline();
			return;
		}
		if (kread((KA_T)i.i_fsptr, (char *)&r, sizeof(r))) {
			(void) sprintf(Namech, "can't read rnode (%#x)",
			    i.i_fsptr);
			printline();
			return;
		}
	}

# if	defined(sgi)
	else if (strcasecmp(Fsinfo[ity].fs_name, "SOC") == 0) {

	/*
	 * Process an SGI socket.
	 */
		printsocket((KA_T)i.i_fsptr);
		return;
	}
# endif	/* sgi */

	else {

	/*
	 * Determine the node type from the inode file type.
	 */
		switch (type) {

		case IFBLK:
			Ntype = N_BLK;
			break;
		case IFCHR:

# if	defined(LACHSOCK)
			if (major(i.i_rdev) == sockdev) {
				printsocket(&i);
				return;
			}
# endif

			Ntype = N_CHR;
			break;
		case IFIFO:
			Ntype = N_FIFO;
			break;
		case IFMPB:
		case IFMPC:
			Ntype = N_MPC;
			break;
		}
	}
/*
 * Obtain lock information.
 */
	if (i.i_filocks) {
		if ((kread((KA_T)i.i_filocks, (char *)&fl, sizeof(fl)))) {
			(void) sprintf(Namech, "can't read filock (%#x)",
				i.i_filocks);
			printline();
			return;
		} else {
			if (fl.set.l_type == F_RDLCK)
				Lock = 'R';
			else if (fl.set.l_type == F_WRLCK)
				Lock = 'W';
			else if (fl.set.l_type == (F_RDLCK | F_WRLCK))
				Lock = '*';
		}
	}
/*
 * Determine the device.
 */
	switch (Ntype) {

		case N_BLK:
		case N_REG:
			Dev = i.i_dev;
			Fdev = 1;
			break;
		case N_CHR:
			Dev = i.i_rdev;
			Fdev = 1;

# if	defined(HASSTREAMS)
			if (printchdevname(Dev, -1)) {
			    if (i.i_sptr) {

			    /*
			     * Follow the stream structure from its stdata
			     * pointer to the module information structure.
			     * Then read the first STRNML-1 characters of
			     * the module name.
			     */
				(void) strcpy(Namech, "STR ");
				if (readstdata((struct stdata *)i.i_sptr, &sd)
				||  readsthead(sd.sd_wrq, &sh)
				||  readstqinit(sh.q_qinfo, &qi)
				||  readstmin(qi.qi_minfo, &mi)
				||  readstidnm(mi.mi_idname, &Namech[4],
					       STRNML-1))
				{
				    (void) sprintf(Namech,
					"no module name for stream at %#x",
					    i.i_sptr);
				} else
				    Namech[STRNML+4-1] = '\0';
			    } else {

# if	defined(eta10)
				(void) strcpy(Namech, "no stream pointer");
# endif

# if	defined(sgi)
				if (strcasecmp(Fsinfo[ity].fs_name,"COM")==0) {
				    Ntype = N_COM;
				    if (printchdevname(Dev, -1))
				        (void) strcpy(Namech,"COM file system");
				} else
				    (void) strcpy(Namech, "no stream pointer");
#endif

			    }
			}

# else	/* !HASSTREAMS */
			(void) printchdevname(Dev, -1);
# endif	/* HASSTREAMS */

			break;
		case N_FIFO:
			prdev = 0;

# if	defined(LACHSOCK)
			if (i.i_nlink == 0 && major(i.i_rdev) == sockdev) {
				(void) sprintf(Devch, " %09x", ia);
				prdev = 1;
			}
# endif

# if	defined(sgi)
			if (major(i.i_dev) == 0x7f
			&&  minor(i.i_dev) == 0xff) {
				(void) sprintf(Devch, "0x%08x", ia);
				prdev = 1;
			}
# endif	/* sgi */

			if ( ! prdev) {
				Dev = i.i_dev;
				Fdev = 1;
			}
			break;
		case N_NFS:
			Dev = m.m_dev;
			Fdev = 1;
			break;
	}
/*
 * Determine the inode number.
 */
	switch (Ntype) {

	case N_NFS:
		Inode = r.r_snfsattr.na_nodeid;
		break;

	case N_BLK:
	case N_CHR:
	case N_COM:
	case N_FIFO:
	case N_REG:
		Inode = i.i_number;
		break;
	}
/*
 * Determine the file size.
 */
	if ( ! Foffset) {
		switch (Ntype) {

		case N_BLK:
			Size = i.i_size;
			Size_def = 1;
			break;
		case N_CHR:
		case N_COM:
			Offset_def = 1;
			break;
		case N_FIFO:
			Offset_def = 1;
			break;
		case N_NFS:
			Size = r.r_snfsattr.na_size;
			Size_def = 1;
			break;
		case N_REG:
			if (type == IFREG || type == IFDIR)
				Size = i.i_size;
				Size_def = 1;
			break;
		}
	} else
		Offset_def = 1;
/*
 * Format the type name.
 */
	switch (type) {

		case IFDIR:
			tn = "DIR";
			break;
		case IFBLK:
			tn = "BLK";
			break;
		case IFCHR:
			tn = "CHR";
			break;
		case IFREG:
			tn = "REG";
			break;
		case IFMPC:
			tn = "MPC";
			break;
		case IFMPB:
			tn = "MPB";
			break;
		case IFIFO:
			tn = "FIFO";
			break;
		case IFLNK:
			tn = "LINK";
			break;
		default:
			(void) sprintf(Type, "%4d", type);
			(void) strcpy(Namech, "unknown inode type");
			tn = NULL;
	}
	if (tn)
		(void) strcpy(Type, tn);
/*
 * Process the file name.
 */
	switch (Ntype) {

	case N_BLK:
	case N_FIFO:
	case N_NFS:
	case N_REG:
		if (Fdev && (lm = srchmounts(Dev)) != NULL) {
			(void) sprintf(Namech, "%s %s%s%s",
				lm->dir    ? lm->dir    : "",
				lm->fsname ? "("        : "",
				lm->fsname ? lm->fsname : "",
				lm->fsname ? ")"        : "");
		}
		break;
	}
/*
 * Test for specified file.
 */
	if (Sfile && Fprint == 0 && Fdev)
		Fprint = isfile(Dev, type, 0, Inode);
	if (Fprint == 0)
		return;
	printline();
}
#endif	/* ! HASVNODES */


#if	defined(_IBMR2)
/*
 * readcdrnode() - read CD-ROM node
 */

int
readcdrnode(ca, c)
	caddr_t ca;			/* cdrnode kernel address */
	struct cdrnode *c;		/* cdrnode buffer */
{
	if (kread((KA_T)ca, (char *)c, sizeof(struct cdrnode))) {
		(void) sprintf(Namech, "can't read cdrnode at %#x", ca);
		return(1);
	}
	return(0);
}
#endif


#if	(defined(_IBMR2) && _AIXV>=30200) || defined(sun)
/*
 * readfifonode() - read fifonode
 */

int
readfifonode(fa, f)
	caddr_t fa;			/* fifonode kernel address */
	struct fifonode *f;		/* fifonode buffer */
{
	if (kread((KA_T)fa, (char *)f, sizeof(struct fifonode))) {
		(void) sprintf(Namech, "can't read fifonode at %#x", fa);
		return(1);
	}
	return(0);
}
#endif


#if	defined(_IBMR2)
/*
 * readgnode() - read gnode
 */

int
readgnode(ga, g)
	caddr_t ga;			/* gnode kernel address */
	struct gnode *g;		/* gnode buffer */
{
	if (kread((KA_T)ga, g, sizeof(struct gnode))) {
		(void) sprintf(Namech, "can't read gnode at %#x", ga);
		return(1);
	}
	return(0);
}
#endif


/*
 * readinode() - read inode
 */

int
readinode(ia, i)
	struct inode *ia;		/* inode kernel address */
	struct inode *i;		/* inode buffer */
{
	if (kread((KA_T)ia, (char *)i, sizeof(struct inode))) {
		(void) sprintf(Namech, "can't read inode at %#x", ia);
		return(1);
	}
	return(0);
}


#if	defined(DHINS)
/*
 * readrnode() - read rnode
 */

int
readrnode(ra, r)
	caddr_t ra;			/* rnode kernel space address */
	struct rnode *r;		/* rnode buffer pointer */
{
	if (kread((KA_T)ra, (char *)r, sizeof(struct rnode))) {
		(void) sprintf(Namech, "can't read rnode at %#x", ra);
		return(1);
	}
	return(0);
}
#endif	/* DHINS */


#if	defined(NeXT) || defined(sun)
/*
 * readsnode() - read snode
 */

int
readsnode(sa, s)
	caddr_t sa;			/* snode kernel space address */
	struct snode *s;		/* snode buffer pointer */
{
	if (kread((KA_T)sa, (char *)s, sizeof(struct snode))) {
		(void) sprintf(Namech, "can't read snode at %#x", sa);
		return(1);
	}
	return(0);
}
#endif


#if	defined(sun)
/*
 * readtnode() - read tmpnode
 */

int
readtnode(ta, t)
	caddr_t ta;			/* tmpnode kernel space address */
	struct tmpnode *t;		/* tmpnode buffer pointer */
{
	if (kread((KA_T)ta, (char *)t, sizeof(struct tmpnode))) {
		(void) sprintf(Namech, "can't read tmpnode at %#x", ta);
		return(1);
	}
	return(0);
}
#endif


#if	defined(HASVNODES)
/*
 * readvnode() - read vnode
 */

int
readvnode(va, v)
	caddr_t va;			/* vnode kernel space address */
	struct vnode *v;		/* vnode buffer pointer */
{
	if (kread((KA_T)va, (char *)v, sizeof(struct vnode))) {
		(void) sprintf(Namech, "can't read vnode at %#x", va);
		return(1);
	}
	return(0);
}
#endif
