/*
 *	print.c - print support functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: print.c,v 1.4 92/10/09 10:27:39 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


/*
 * endnm() - locate end of Namech
 */

char *
endnm()
{
	char *s;

	for (s = Namech; *s; s++)
		;
	return(s);
}


/*
 * printfile() - print file information
 */

void
printfile(fp)
	KA_T fp;			/* kernel file table address */
{
	struct file f;
	int flag;

#if	defined(eta10)
	if (fp < Ftaddr || fp >= Ftaddrl) {
		(void) sprintf(Namech,
			"file pointer (%#x) out of range [%#x,%#x]",
			fp, Ftaddr, Ftaddrl);
		printline();
		return;
	}
#endif

	if (kread(fp, (char *) &f, sizeof(f))) {
		(void) sprintf(Namech, "can't read file struct from %#x", fp);
		printline();
		return;
	}
	Offset = f.f_offset;

	if (f.f_count) {

	/*
	 * Construct access code.
	 */
		if ((flag = (f.f_flag & (FREAD | FWRITE))) == FREAD)
			Access = 'r';
		else if (flag == FWRITE)
			Access = 'w';
		else if (flag == (FREAD | FWRITE))
			Access = 'u';

#if	defined(HASFTYPE)
	/*
	 * Process structure by its type.
	 */
		switch (f.f_type) {

		case DTYPE_VNODE:
			printnode(f.f_data);
			break;

		case DTYPE_SOCKET:
			Offset_def = 1;
			printsocket((caddr_t)f.f_data);
			break;

# if	defined(DTYPE_GNODE)
		case DTYPE_GNODE:
			printdevice(f.f_data);
			break;
# endif

		default:

# if	defined(hpux)
			if (f.f_type != 0
			||  (f.f_ops != NULL && f.f_ops != Vnfops))
# endif

# if	defined(DINS)
			if (f.f_type != 0 || f.f_ops != NULL)
# endif


			{
				(void) sprintf(Namech,
					"%#x file struct, ty=%#x, op=%#x",
					fp, f.f_type, f.f_ops);
			}
			printline();
		}
#else	/* !HASFTYPE */

# if	defined(eta10) || defined(sgi)
		printnode(f.f_inode);
# endif

#endif	/* HASFTYPE */

	}
}


/*
 * printinaddr() - print Internet address
 */

void
printinaddr(ia, p)
	struct in_addr *ia;		/* Internet address */
	u_short p;			/* port */
{
	unsigned char *u;

	if (ia->s_addr == INADDR_ANY) {
		(void) sprintf(endnm(), "*:%d", p);
		return;
	}
	u = (unsigned char *) ia;
	(void) sprintf(endnm(), "%u.%u.%u.%u:%u", u[0], u[1], u[2], u[3], p);
}


/*
 * printiproto() - print Internet protocol name
 */

void
printiproto(p)
	int p;				/* protocol number */
{
	int i;
	static int m = -1;
	char *s;

	switch(p) {

#if	defined(IPPROTO_TCP)
	case IPPROTO_TCP:
		s = "TCP";
		break;
#endif

#if	defined(IPPROTO_UDP)
	case IPPROTO_UDP:
		s = "UDP";
		break;
#endif

#if	defined(IPPROTO_IP)
	case IPPROTO_IP:
		s = "IP";
		break;
#endif

#if	defined(IPPROTO_ICMP)
	case IPPROTO_ICMP:
		s = "ICMP";
		break;
#endif

#if	defined(IPPROTO_IGMP)
	case IPPROTO_IGMP:
		s = "IGMP";
		break;
#endif

#if	defined(IPPROTO_GGP)
	case IPPROTO_GGP:
		s = "GGP";
		break;
#endif

#if	defined(IPPROTO_EGP)
	case IPPROTO_EGP:
		s = "EGP";
		break;
#endif

#if	defined(IPPROTO_PUP)
	case IPPROTO_PUP:
		s = "PUP";
		break;
#endif

#if	defined(IPPROTO_IDP)
	case IPPROTO_IDP:
		s = "IDP";
		break;
#endif

#if	defined(IPPROTO_ND)
	case IPPROTO_ND:
		s = "ND";
		break;
#endif

#if	defined(IPPROTO_RAW)
	case IPPROTO_RAW:
		s = "RAW";
		break;
#endif

#if	defined(IPPROTO_HELLO)
	case IPPROTO_HELLO:
		s = "HELLO";
		break;
#endif

#if	defined(IPPROTO_PXP)
	case IPPROTO_PXP:
		s = "PXP";
		break;
#endif

#if	defined(IPPROTO_RAWIP)
	case IPPROTO_RAWIP:
		s = "RAWIP";
		break;
#endif

#if	defined(IPPROTO_RAWIF)
	case IPPROTO_RAWIF:
		s = "RAWIF";
		break;
#endif

	default:
		s = NULL;
	}
	if (s)
		(void) sprintf(Iproto, "%*.*s", IPROTOL, IPROTOL, s);
	else {	
		if (m < 0) {
			for (i = 0, m = 1; i < IPROTOL-1; i++)
				m *= 10;
		}
		(void) sprintf(Iproto, "%d?", p % m);
	}
}


/*
 * printline() - print output line
 */

void
printline()
{
	static int lpid = -1;

	if (Fprint == 0)
		return;
	if (Fterse) {
		if (Pid != lpid) {
			(void) printf("%d\n", Pid);
			lpid = Pid;
		}
		return;
	}
	if (Hdr == 0) {

#if	defined(DEINSSG)
	    (void) printf("COMMAND     PID     USER   FD   TYPE     DEVICE ");
#endif

#if	defined(hpux)
	    (void) printf("COMMAND     PID     USER   FD   TYPE       DEVICE ");
#endif

	    if (Fsize)
		(void) printf("%10s", Foffset ? "OFFSET" : "SIZE/OFF");
	    (void)printf("  INODE/NAME\n");
	}
	Hdr++;
	(void) printf("%-9.9s%6d %8s %4s%c%c %4.4s ",
			Command ? Command : "",
			Pid,
			User,
			Fd,
			Access,
			Lock,
			Type
	);
	if (Fdev) {

#if	defined(_IBMR2)
# if	_AIXV>=30200
		if (Dev & SDEV_REMOTE)
			(void) printf("  NFS %4d ", minor(Dev));
		else
			(void) printf(" %4d,%4d ", major(Dev), minor(Dev));
# else	/* _AIXV<30200 */
		(void) printf(" %4d,%4d ", major(Dev), minor(Dev));
# endif
#endif

#if	defined(DENSSG)
		(void) printf(" %4d,%4d ", major(Dev), minor(Dev));
#endif

#if	defined(hpux)
		(void) printf("%3d,0x%06x ", major(Dev), minor(Dev));
#endif

	} else {
		if (Devch[0])

#if	defined(DEINSSG)
			(void) printf("%s ", Devch);
		else
			(void) printf("           ");
#endif

#if	defined(hpux)
			(void) printf("  %s ", Devch);
		else
			(void) printf("             ");
#endif

	}
	if (Fsize) {
		if (Size_def)
		    (void) printf("%10u", Size);
		else if (Offset_def)
		    (void) printf("%#10x", Offset);
		else
		    (void) printf("          ");
	}
	if (Inode >= 0)
		(void) printf("%7ld ", Inode);
	else if (Inode == -2l) {
		if (Iproto[0] == '\0')
			(void) printf("%7s ", "");
		else
			(void) printf("%7.7s ", Iproto);
	} else
		(void) printf("%7s ", "");
	(void) printf("%s\n", Namech);
}


/*
 * printprep() - prepare global variables for printing a line
 */

void
printprep(c, d, f, u)
	char *c;			/* command name */
	char *d;			/* file descriptor (NULL if none) */
	int f;				/* print flag value */
	uid_t u;			/* user ID number */
{
	Access = ' ';
	Command = c;
	Devch[0] = Iproto[0] = Type[0] = '\0';
	if (d) {
		(void) strncpy(Fd, d, FDLEN - 1);
		Fd[FDLEN - 1] = '\0';
	}
	Fdev = 0;
	Fprint = f;
	Inode = -1l;
	Lock = ' ';
	Namech[0] = '\0';
	Offset_def = Size_def = 0;
	Ntype = N_REG;
	printuid(u);
}


/*
 * printuid() - print User ID or login name
 */

void
printuid(u)
	uid_t u;			/* User ID number */
{
	static int p = 0;
	static uid_t pu;
	struct passwd *pw;

	if (p && u == pu)
		return;
	p = 1;
	pu = u;
	if (Futol && (pw = getpwuid(u)) != NULL)
		(void) sprintf(User, "%*.*s", USERPRTL, USERPRTL, pw->pw_name);
	else
		(void) sprintf(User, "%*d", USERPRTL, u);
}
