/*
 *	sock.c - socket printing functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: sock.c,v 1.5 92/10/09 10:27:41 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


#if	defined(BSDSOCK)
/*
 * printsocket() - print BSD socket
 */

void
printsocket(sa)
	caddr_t sa;			/* socket address in kernel */
{
	int fam;

# if	defined(DINS) || defined(HPUX8)
	struct domain d;
# endif

# if	defined(DHINS)
	struct mbuf mb;
	struct sockaddr_un *ua = NULL;
	struct sockaddr_un un;
	struct unpcb uc, unp;
# endif

# if	defined(_IBMR2)
	struct gnode g;
	struct inode i;
	int is = 0;
	struct vnode v;
# endif

	struct inpcb inp;
	struct protosw p;
	struct socket s;

        (void) strcpy(Type, "sock");
	Inode = -2l;
	if (sa == NULL) {
		(void) sprintf(Namech, "no socket address");
		printline();
		return;
	}
        if (kread((KA_T) sa, (char *) &s, sizeof(s))) {
                (void) sprintf(Namech, "can't read socket struct from %#x",
			sa);
                printline();
                return;
        }

# if	defined(DHINS)
	if ( ! s.so_type) {
                (void) strcpy(Namech, "no socket type");
                printline();
                return;
	}
        if (s.so_proto == NULL
	||  kread((KA_T) s.so_proto, (char *) &p, sizeof(p))) {
                (void) strcpy(Namech, "no protocol switch");
                printline();
                return;
        }
# endif

# if	defined(DINS) || defined(HPUX8)
        if (kread((KA_T) p.pr_domain, (char *) &d, sizeof(d))) {
                (void) sprintf(Namech, "can't read domain struct from %#x",
                        p.pr_domain);
                printline();
                return;
        }
# endif

/*
 * Process socket by the associated domain family.
 */

# if	defined(DINS) || defined(HPUX8)
	switch ((fam = d.dom_family))
# endif

# if	defined(HPUX7)
	switch ((fam = p.pr_family))
# endif

	{
	case AF_INET:
		if (Fnet)
			Fprint = 1;
		(void) strcpy(Type, "inet");

# if	defined(DHINS)
	/*
	 * Read Internet protocol control block.
	 */
		if (s.so_pcb == NULL
		|| kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp)))
			return;
		if ((struct socket *)sa != inp.inp_socket)
			return;
		(void) sprintf(Devch, "0x%08x",
			      (inp.inp_ppcb == NULL) ? s.so_pcb
						     : inp.inp_ppcb);

# endif

		printiproto(p.pr_protocol);

# if	defined(DYNIX)
		printinaddr(&inp.inp_laddr, ntohs(inp.inp_lport));
# endif

# if	defined(HINS)
		printinaddr(&inp.inp_laddr, inp.inp_lport);
# endif

		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
			(void) strcat(endnm(), "->");

# if	defined(DYNIX)
			printinaddr(&inp.inp_faddr, ntohs(inp.inp_fport));
# endif

# if	defined(HINS)
			printinaddr(&inp.inp_faddr, inp.inp_fport);
# endif

		}
		break;

	case AF_UNIX:
		if (Funix)
			Fprint = 1;
		(void) strcpy(Type, "unix");
	/*
	 * Read Unix protocol control block and the Unix address structure.
	 */

		(void) sprintf(Devch, "0x%08x", sa);

# if	defined(DHINS)
		if (kread((KA_T) s.so_pcb, (char *) &unp, sizeof(unp))) {
			(void) sprintf(Namech, "can't read unpcb at %#x",
				s.so_pcb);
			break;
		}
		if ((struct socket *)sa != unp.unp_socket) {
			(void) sprintf(Namech, "unp_socket (%#x) mismatch",
				unp.unp_socket);
			break;
		}
		if (unp.unp_addr) {
		    if (kread((KA_T) unp.unp_addr, (char *) &mb, sizeof(mb))) {
			(void) sprintf(Namech,
				"can't read unp_addr at %#x",
				unp.unp_addr);
			break;
		    }
# endif

# if	defined(_IBMR2)

#  if	_AIXV>=30200
		    ua = (struct sockaddr_un *) ((char *) &mb +
			 (mb.m_hdr.mh_data - (caddr_t) unp.unp_addr));
#  endif

#  if	_AIXV<30200
		    ua = (struct sockaddr_un *)(((char *)&mb) + mb.m_off);
#  endif
# endif

# if	defined(DHNS)
		    ua = (struct sockaddr_un *)(((char *)&mb) + mb.m_off);
# endif

# if	defined(DHINS)
		}
		if (ua == NULL) {
			ua = &un;
			(void) bzero((char *)ua, sizeof(un));
			ua->sun_family = AF_UNSPEC;
		}
	/*
	 * Print information on Unix socket that has no address bound
	 * to it, although it may be connected to another Unix domain
	 * socket as a pipe.
	 */
		if (ua->sun_family != AF_UNIX) {
			if (ua->sun_family == AF_UNSPEC) {
				if (unp.unp_conn) {
					if (kread((KA_T) unp.unp_conn,
						(char *) &uc, sizeof(uc))) {
					    (void) sprintf(Namech,
						"can't read unp_conn at %#x",
						unp.unp_conn);
					} else {
					    (void) sprintf(Namech,
						"->0x%08x", uc.unp_socket);
					}
				} else
					(void) strcpy(Namech, "->(none)");
			} else
				(void) sprintf(Namech,
					"unknown sun_family (%d)",
					ua->sun_family);
			break;
		}
# endif	/* DHINS */

# if	defined(_IBMR2)
	/*
	 * Read any associated vnode and then read its gnode and inode.
	 */
		g.gn_type = VSOCK;
		if (unp.unp_vnode && readvnode(unp.unp_vnode, &v) == 0) {
			if (v.v_gnode && readgnode(v.v_gnode, &g) == 0) {
				Lock = isglocked(&g);
				if (g.gn_type == VSOCK && g.gn_data
				&& readinode(g.gn_data, &i) == 0)
					is = 1;
			}
		}
	/*
	 * Print Unix socket information.
	 */
		if (is) {
			Dev = i.i_dev;
			Fdev = 1;
			Devch[0] = '\0';
			Inode = (long)i.i_number;
		}
# endif	/* _IBMR2 */

# if	defined(DHINS)
		(void) sprintf(Namech, "%.*s",
			mb.m_len - sizeof(ua->sun_family),
			ua->sun_path[0] ? ua->sun_path : "no address");
		break;
# endif

	default:
		(void) sprintf(Namech, "unknown family %#x", fam);
	}

	printline();
}
#endif	/* BSDSOCK */


#if	defined(LACHSOCK)
/*
 * printsocket() - print Lachman System V socket information
 */

void
printsocket(i)
	struct inode *i;		/* inode */
{
	struct inpcb cb;
	struct domain d;
	KA_T ka;
	struct queue q;
	struct socket s;
	struct socket *sp = NULL;
	struct stdata std;

	(void) strcpy(Type, "inet");

	if (Fnet)
		Fprint = 1;
	(void) strcpy(Type, "inet");
/*
 * Read the socket pointer from the kernel's socktab[], then the socket.
 */
	ka = Socktab + (KA_T)(devtosock(i->i_rdev) * sizeof(struct socket *));
	if (kread(ka, (char *)&sp, sizeof(sp)) || sp == NULL
	||  sp == SOCK_RESERVE) {
		(void) sprintf(Namech, "bad socket pointer (%#x) at %#x",
			sp, ka);
		printline();
		return;
	}
	if (kread((KA_T)sp, (char *)&s, sizeof(s))) {
		(void) sprintf(Namech, "can't read socket from %#x", sp);
		printline();
		return;
	}
	if ( ! s.so_type) {
		(void) sprintf(Namech, "no socket type");
		printline();
		return;
	}
/*
 * Read the domain structure.
 */
	if (s.so_proto.pr_domain == NULL
	||  kread((KA_T)s.so_proto.pr_domain, (char *)&d, sizeof(d))) {
		(void) strcpy(Namech, "no domain for socket");
		printline();
		return;
	}
/*
 * Process Internet domain socket.
 */
	if (d.dom_family == AF_INET) {
		printiproto(s.so_proto.pr_protocol);
		Inode = -2l;
	/*
	 * Read the stream's stdata structure and follow it's write queue
	 * pointer set to the end of the stream.  The Internet protocol
	 * control block pointer should be found there.
	 */
		if (readstdata((struct stdata *)i->i_sptr, &std)) {
			printline();
			return;
		}
		if (readsthead(std.sd_wrq, &q)) {
			printline();
			return;
		}
		if (q.q_ptr == NULL
		||  kread((KA_T)q.q_ptr, (char *)&cb, sizeof(cb))) {
			(void) strcpy(Namech, "no inpcb available");
		} else {
			(void) sprintf(Devch, " %09x", q.q_ptr);
			printinaddr(&cb.inp_laddr, cb.inp_lport);
			if (cb.inp_faddr.s_addr != INADDR_ANY
			|| cb.inp_fport != 0) {
				(void) strcat(endnm(), "->");
				printinaddr(&cb.inp_faddr, cb.inp_fport);
			}
		}
	} else {
/*
 * Printing for this domain is not supported.
 */
		(void) sprintf(Namech, "unknown domain family (%d)",
			d.dom_family);
	}
	printline();
}
#endif	/* LACHSOCK */


#if	defined(sgi)
/*
 * printsocket() - print SGI socket
 */

void
printsocket(sa)
	KA_T sa;		/* socket address */
{
	struct inpcb cb;
	struct domain d;
	struct inode i;
	int is;
	struct mbuf m;
	struct protosw p;
	struct socket s;
	struct unpcb u, uc;
	struct sockaddr_un *ua = NULL;
	struct sockaddr_un un;
/*
 * Read the socket, then the protocol switch structure, then the
 * domain structure.
 */
	if (kread(sa, (char *)&s, sizeof(s))) {
		(void) sprintf(Namech, "can't read socket structure at %#x",
			sa);
		printline();
		return;
	}
	if (s.so_proto == NULL
	||  kread((KA_T)s.so_proto, (char *)&p, sizeof(p))) {
		(void) sprintf(Namech,
			"can't read protosw (%#x) for socket %#x",
			sa, s.so_proto);
		printline();
		return;
	}
	if (p.pr_domain == NULL
	||  kread((KA_T)p.pr_domain, (char *)&d, sizeof(d))) {
		(void) sprintf(Namech,
			"can't read domain (%#x) for protosw at %#x",
			p.pr_domain, s.so_proto);
		printline();
		return;
	}
	switch (d.dom_family) {

	/*
	 * Process Unix domain socket.
	 */

	case AF_UNIX:
		if (Funix)
			Fprint = 1;
		(void) strcpy(Type, "unix");
		(void) sprintf(Devch, "0x%08x", sa);
	/*
	 * Read the Unix protocol control block.
	 */
		if (kread((KA_T) s.so_pcb, (char *)&u, sizeof(u))) {
			(void) sprintf(Namech, "can't read unpcb at %#x",
				s.so_pcb);
			break;
		}
		if ((struct socket *)sa != u.unp_socket) {
			(void) sprintf(Namech, "unp_socket (%#x) mismatch",
				u.unp_socket);
			break;
		}
	/*
	 * If there is an address, read it.
	 */
		if (u.unp_addr) {
		    if (kread((KA_T) u.unp_addr, (char *)&m, sizeof(m))) {
			(void) sprintf(Namech, "can't read unp_addr at %#x",
				u.unp_addr);
			break;
		    }
		    ua = (struct sockaddr_un *)(((char *)&m) + m.m_off);
		}
		if (ua == NULL) {
			ua = &un;
			(void) bzero((char *)ua, sizeof(un));
			ua->sun_family = AF_UNSPEC;
		}
	/*
	 * Get information on Unix socket that has no address bound
	 * to it, although it may be connected to another Unix domain
	 * socket as a pipe.
	 */
		if (ua->sun_family != AF_UNIX) {
			if (ua->sun_family == AF_UNSPEC) {
				if (u.unp_conn) {
					if (kread((KA_T) u.unp_conn,
						(char *) &uc, sizeof(uc))) {
					    (void) sprintf(Namech,
						"can't read unp_conn at %#x",
						u.unp_conn);
					} else {
					    (void) sprintf(Namech,
						"->0x%08x", uc.unp_socket);
					}
				} else
					(void) strcpy(Namech, "->(none)");
			} else
				(void) sprintf(Namech,
					"unknown sun_family (%d)",
					ua->sun_family);
			break;
		}
	/*
	 * Read associated inode.
	 */
		if (u.unp_inode && readinode(u.unp_inode, &i) == 0)
			is = 1;
		else
			is = 0;
	/*
	 * Generate Unix socket information.
	 */
		if (is) {
			Dev = i.i_dev;
			Fdev = 1;
			Devch[0] = '\0';
			Inode = (long)i.i_number;
		}
		(void) sprintf(Namech, "%.*s",
			m.m_len - sizeof(ua->sun_family),
			ua->sun_path[0] ? ua->sun_path : "no address");
		break;
	/*
	 * Process Internet domain socket.
	 */

	case AF_INET:
		if (Fnet)
			Fprint = 1;
		(void) strcpy(Type, "inet");
		printiproto(p.pr_protocol);
		Inode = -2l;
		if (s.so_pcb == NULL
		||  kread((KA_T)s.so_pcb, (char *)&cb, sizeof(cb))) {
			(void) sprintf(Namech, "no inpcb available");
		} else {
			(void) sprintf(Devch, "0x%08x",
				cb.inp_ppcb != NULL ? cb.inp_ppcb
						    : s.so_pcb);
			if (p.pr_protocol == IPPROTO_TCP
			||  p.pr_protocol == IPPROTO_UDP) {
			    printinaddr(&cb.inp_laddr, cb.inp_lport);
			    if (cb.inp_faddr.s_addr != INADDR_ANY
			    ||  cb.inp_fport != 0) {
				(void) strcat(endnm(), "->");
				printinaddr(&cb.inp_faddr, cb.inp_fport);
			    }
			}
		}
		break;
	default:
		(void) sprintf(Namech, "unknown domain family (%d)",
			d.dom_family);
	}
	printline();
}
#endif	/* sgi */
