/*
 *	store.c - global storage for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: store.c,v 1.6 92/09/11 11:44:07 abe Exp $";
#endif



#include "lsof.h"


/*
 * Global storage definitions
 */

char Access;			/* f_flag access code: `R' = read
				 *		       `W' = write
				 *		       `U' = read and write */

#if	defined(sgi) || defined(sun)
struct clone *Clone = NULL;
#endif

char *Command;			/* command name */
char *Core = NULL;		/* core file path */
dev_t Dev;			/* device number */
char Devch[32];			/* alternate characters for device printing */

struct dev *Devtp = NULL;	/* device table pointer */

char Fd[FDLEN];			/* file descriptor for printing */
int Fdev = 0;			/* have device number flag */
int Fdevmsg = 0;		/* -d flag status */
int Fhelp = 0;			/* -h flag status */

#if	defined(_IBMR2)
int Flibdup = 1;		/* -L flag status */
#endif

int Fnet = 0;			/* -n flag status */
int Fnfs = 0;			/* -N flag status */
int Foffset = 0;		/* -o flag status */
int Fprint = 0;			/* print line flag */
int Fsize = 1;			/* -s flag status */

#if	defined(eta10) || defined(sgi)
struct fsinfo *Fsinfo = NULL;	/* kernel's fsinfo[], indexed by fstyp */
KA_T Ftaddr;			/* file table kernel address */
KA_T Ftaddrl;			/* file table kernel address limit */
#endif

int Fterse = 0;			/* -t flag status */
int Funix = 0;			/* -U flag status */
int Futol = 1;			/* -l flag status */
int Hdr = 0;			/* header print status */

long Inode = -1l;		/* inode number
				 *	-1: none (don't print anything)
				 *	-2: blank (print spaces) */
char Iproto[IPROTOL+1];		/* Internet protocol name */

#if	defined(sun)
kvm_t *Kd;			/* kvm descriptor */
#endif

#if	defined(DEHINSG)
int Kmem = -1;			/* /dev/kmem file descriptor */
#endif

#if	defined(HASVNODES)
struct l_vfs *Lvfs = NULL;			/* local vfs structure table */
#endif

#if	defined(eta10) || defined(HPUX7) || defined(_IBMR2)
int Mem = -1;			/* /dev/mem file descriptor */
#endif

char Lock = ' ';		/* lock status */

struct mounts *Mtab = NULL;	/* mounted devices */

int Mxpid = 0;			/* maximum PID table entries */
int Mxuid = 0;			/* maximum UID table entries */

#if	defined(HASSECURITY)
int Myuid;			/* real UID of this lsof process */
#endif

char Namech[1024];		/* name characters for printing */
int Ndev = 0;			/* number of device table entries */

#if	defined(eta10) || (sgi)
short Nfstyp = 0;		/* number of fstypsw[] entries */
#endif

#if	defined(DYNIX)
struct nlist Nl[] = {
        { "_proc" },
        {"_nproc"},
        {"_nfs_vnodeops"},
	{"_Sysmap"},
	{ "" },
};
#endif

#if	defined(eta10)
struct nlist Nl[] = {
	{ "file" },
	{ "fsinfo" },
	{ "nfstyp" },
	{ "proc" },
	{ "v" },

# if	defined(LACHSOCK)
	{ "sockdev" },
	{ "socktab" },
#endif

	{ "" },
};
#endif	/* eta10 */


#if	defined(hpux)
struct nlist Nl[] = {

# if	defined(hp9000s300)
        {"_proc"},
        {"_nfs_vnodeops"},
        {"_nproc"},
        {"_vnodefops"},

#  if	defined(HPUX7)
	{"_Usrptmap"},
	{"_usrpt"},
#  endif
# endif

# if	defined(hp9000s800)
	{"proc"},
	{"nproc"},
        {"nfs_vnodeops"},
        {"vnodefops"},

#  if	defined(HPUX7)
	{"ubase"},
	{"npids"},
#  endif
# endif

	{ "" }
};
#endif	/* hpux */

#if	defined(_IBMR2)
/*
 * Note: the IBMR2 name list structure cannot be expanded in the
 *       coventional fashion.  Consult the knlist() man page.
 */

struct nlist Nl[] = { "u" };
#endif	/* _IBMR2 */


#if	defined(NeXT)

struct nlist Nl[] = {
        { {"_allproc"},      0, 0, 0 },
        { {"_nfs_vnodeops"}, 0, 0, 0},
        {  {""},             0, 0, 0  },
};
#endif

#if	defined(sgi)
struct nlist Nl[] = {
	{ "file" },
	{ "fsinfo" },
	{ "nfstyp" },
	{ "proc" },
	{ "v" },
	{ "" }
};
#endif

#if	defined(sun)
struct nlist Nl[] = {
        {"_nfs_vnodeops"},
	{ "_tmp_vnodeops"},
	{ "" }
};
#endif

char *Nmlst = NULL;		/* namelist file path */

#if	defined(DYNIX)
int Notkmem = 0;		/* core file is not /dev/kmem */
#endif

int Npid = 0;			/* -p flag count */
int Ntype;			/* node type (see N_* symbols) */
int Nuid = 0;			/* -u flag count */
off_t Offset;			/* file offset from file structure */
int Offset_def;			/* file offset is defined */
int Pid;			/* Process ID */
char *Pn;			/* program name */

#if	defined(HPUX7) && defined(hp9000s800)
int npids;			/* number of PIDs (for uvadd()) */
struct proc *proc;		/* process table address (for uvadd()) */
#endif

#if	defined(hpux)
long Procaddr;			/* kernel process table entry address */
#endif

struct dev **Sdev = NULL;	/* pointer to device table pointers, sorted
				 * by device */

struct sfile *Sfile = NULL;	/* chain of files to search for */

off_t Size;			/* file size */
int Size_def;			/* file size is defined */
int *Spid = NULL;		/* Process IDs to search for */

#if	defined(LACHSOCK)
int sockdev;			/* socket major device number (lower case
				 * name required by devtosock() macro) */
KA_T Socktab;			/* kernel's socktab[] address */
#endif

int *Suid = NULL;		/* User IDs to search for */

#if	defined(DYNIX) || defined(hpux)
int Swap = -1;			/* swap device file descriptor */
#endif

#if	defined(DYNIX)
struct pte *Sysmap;		/* kernel _Sysmap address */
#endif

jmp_buf Tmo_jbuf;		/* jump buffer for mount stat timeout */

char Type[8];			/* type for printing */

char User[USERPRTL+1];		/* User ID */

#if	defined(HPUX7) && defined(hp9000s800)
struct user *ubase;		/* user area base (for uvadd()) */
#endif

#if	defined(HPUX7) && defined(hp9000s300)
struct user *ubase;		/* user area base (for uvadd()) */
struct pte *Usrptmap;		/* user page table map pointer */
struct pte *usrpt;		/* user page table pointer
				 * (for bktomx from vmmac.h) */
#endif

#if	defined(eta10) || defined(sgi)
struct var Var;			/* kernel's variables */
#endif

#if	defined(hpux)
struct fileops *Vnfops;		/* HP-UX vnodefops switch address */
#endif
