/*
 *	text.c - text access functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: text.c,v 1.1 92/08/25 10:59:51 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


#if	defined(_IBMR2)
/*
 * getle() - get loader entry structure
 */

static struct le *
getle(a, u, sid)
	long a;				/* loader entry kernel address */
	struct user *u;			/* load user structure address */
	long sid;			/* user structure sedment ID */
{
	static struct le le;

	if (a < Nl[0].n_value)
		return(NULL);
	if (((char *)a + sizeof(le)) <= ((char *)Nl[0].n_value + U_SIZE))
		return((struct le *)((char *)u + a - Nl[0].n_value));
	if (kreadx((off_t)a, (char *)&le, sizeof(le), sid) == 0)
		return(&le);
	return(NULL);
}


/*
 * printtext() - print text access information
 */

void
printtext(pflag, u, uid, sid)
	int pflag;			/* print status flag */
	struct user *u;			/* user area pointer */
	uid_t uid;			/* user ID number */
	long sid;			/* user area segment ID */
{

	static struct file **f = NULL;
	char fd[FDLEN];
	int i, j, n;
	struct le *le;
	long ll;
	static MALLOC_S nf = 0;

/*
 * Display information on the exec'd entry.
 */
	if ((le = getle(u->u_loader[1], u, sid)) != NULL && le->fp) {
		printprep(u->u_comm, " txt", pflag, uid);
		printfile((KA_T)le->fp);
	}
/*
 * Display the loader list.
 */
	for (i = n = 0, ll = u->u_loader[0]; ll; ll = (long)le->next) {
		(void) sprintf(fd, " L%02d", i);
		printprep(u->u_comm, fd, pflag, uid);
		if ((le = getle(ll, u, sid)) == NULL) {
			(void) sprintf(Namech, "can't read loader entry at %#x",
				ll);
			printline();
			return;
		}
	/*
	 * Skip the exec'd file and entries with no file pointers.
	 */
		if (ll == u->u_loader[1] || ! le->fp)
			continue;
		if (Flibdup) {

		/*
		 * Skip entries with a file pointer that has already been
		 * displayed.  Record new, unique file pointers.
		 */
		    for (j = 0; j < n; j++) {
			if (f[j] == le->fp)
			    break;
		    }
		    if (j < n)
			continue;
		    if (f == NULL) {
			if ((f = (struct file **)malloc((MALLOC_S)
			   (sizeof(struct file *) * 10)))
			== NULL) {
			    (void) fprintf(stderr,
				"%s: no space for text file pointers\n", Pn);
			    exit(1);
			}
			nf = 10;
		    } else if (n >= nf) {
			if ((f = (struct file **)realloc((MALLOC_P *)f,
				 (MALLOC_S)((nf + 10) * sizeof(struct file *))))
			== NULL) {
			    (void) fprintf(stderr,
				"%s: no more space for text file pointers\n",
				Pn);
			    exit(1);
			}
			nf += 10;
		    }
		    f[n++] = le->fp;
		}
	/*
	 * Display the loader entry.
	 */
		printfile((KA_T)le->fp);
		i++;
	}
}
#endif	/* _IBMR2 */
