/*
 *	user.c - user area functions for lsof
 */


/*
 * Copyright 1992 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1991, 1992 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: user.c,v 1.4 92/09/05 13:47:31 abe Exp $";
#endif


#include "lsof.h"
#include "proto.h"


#if	defined(DYNIX)
/*
 * getu() - get DYNIX user area
 */

int
getu(p, u)
	struct proc *p;			/* process area pointer */
	struct user *u;			/* user area destination */
{
	off_t sp;

	if ((p->p_flag & SLOAD) == 0) {
		if (Swap < 0)
			return(1);
		sp = (off_t) dtob(p->p_swaddr);
		if (lseek(Swap, sp, L_SET) != sp
		||  read(Swap, (char*)u, U_SIZE) != U_SIZE)
			return(1);
	} else {
		if (kread((KA_T)p->p_uarea, (char *)u, U_SIZE))
			return(1);
	}
	return(0);
}
#endif	/* DYNIX */


#if	defined(eta10)
/*
 * getu() - get eta10 user area
 */

struct user *
getu(p, pa)
	struct proc *p;			/* process structure pointer */
	KA_T pa;			/* process' kernel address */
{
	char *cp, *cp1;
	int i;
	struct pfdat pd;
	static struct user *u;
	static char *ua = NULL;
	static MALLOC_S ual;

/*
 * Allocate static user area buffer.
 */
	if (ua == NULL) {
		ual = (MALLOC_S) (sizeof(struct user) 
		    + ((Var.v_nofiles - 1) * sizeof(struct file *)));
		if ((ua = malloc(ual)) == NULL) {
			(void) fprintf(stderr,
				"%s: no space for %d byte user area\n",
				Pn, ual);
			exit(1);
		}
		u = (struct user *)ua;
	}
/*
 * Read user area via page table pointers in proc structure.
 */
	for(i = 0, cp = ua, cp1 = ua + ual; cp < cp1; i++, cp += NBPP) {
		if (mread((KA_T)pg_getpfd(&(p->p_ubptbl[i])), (char *)&pd,
		    sizeof(pd)))
			return(NULL);
		if (mread((KA_T)(pd.pf_caddr), cp, (cp1-cp)
		>   NBPP ? NBPP : (READLEN_T)(cp1-cp)))
			return(NULL);
	}
	if (u->u_procp != (struct proc *)pa)
		return(NULL);
	return(u);
}
#endif	/* eta10 */


#if	defined(hpux)
/*
 * getu() - get HP-UX user area
 */

int
getu(p, u)
	struct proc *p;			/* process area pointer */

# if	defined(HPUX7)
	char *u;			/* user area destination */
# endif

# if	defined(HPUX8)
	struct user *u;			/* user area destination */
# endif

{

# if	defined(HPUX7)
	long sw;

#  if	defined(hp9000s300)
	struct pte pte1, pte2;
	off_t pte_off, pte_addr;
#  endif
# endif

# if	defined(HPUX8)
	char *c, *s;
	int i;
	struct pst_status ps;
# endif

# if	defined(HPUX7)
/*
 * Read the HP-UX 7.x user area from the swap file or memory.
 */
	if ((p->p_flag & SLOAD) == 0) {

	/*
	 * If the process is not loaded, read the user area from the swap file.
	 */
		if (Swap < 0)
			return(1);
		sw = (long)p->p_swaddr;

#  if	defined(hp9000s800)
		sw += (long)ctod(btoc(STACKSIZE * NBPG));
#  endif

		if (lseek(Swap, (off_t)dtob(sw), L_SET) == (off_t)-1
		||  read(Swap, u, U_SIZE) != U_SIZE)
			return(1);
		return(0);
	}

/*
 * Read the HP-UX 7.x user area via the page table.
 */

#  if	defined(hp9000s300)
	pte_off = (off_t) &Usrptmap[btokmx(p->p_p0br) + p->p_szpt - 1];
	if (kread(pte_off, (char *)&pte1, sizeof(pte1)))
		return(1);
	pte_addr = (KA_T) (ctob(pte1.pg_pfnum + 1)
		 - ((UPAGES + FLOAT) * sizeof(pte2)));
	if (mread((KA_T)pte_addr, (char *)&pte2, sizeof(pte2)))
		return(1);
	if (mread((KA_T)ctob(pte2.pg_pfnum), u, sizeof(struct user)))
		return(1);
#  endif

#  if	defined(hp9000s800)
	if (kread((KA_T)uvadd((struct proc *)Procaddr),u,sizeof(struct user)))
		return(1);
#  endif
# endif	/* HPUX7 */

# if	defined(HPUX8)
/*
 * Use the undocumented HP-UX 8.x pstat() syscall to read process status.
 *
 * (HP won't release pstat(2) documentation to customers who have paid for
 * the inclusion of the syscall in HP-UX.  You figure that one out.)
 */
	(void) bzero(u, U_SIZE);
	if (pstat(PSTAT_PROC, &ps, sizeof(ps), 0, p->p_pid) != 1) {
		(void) fprintf(stderr, "%s: can't pstat process %d: %s\n",
			Pn, p->p_pid, sys_errlist[errno]);
		return(1);
	}
/*
 * Scan the pst_cmd command buffer and skip to the last component of the
 * first path name.  Also skip any leading `-', signifying a login shell.
 * Copy the result to u_comm[].
 */
	c = ps.pst_cmd;
	ps.pst_cmd[PST_CLEN - 1] = '\0';	/* paranoia */
	if (*c == '/' || *c == '.') {
		for (s = c; *s && *s != ' '; s++) {
			if (*s == '/')
				c = s + 1;
		}
	}
	if (*c == '-')
		c++;
	for (i = 0; i < MAXCOMLEN; i++) {
		if (*c == '\0' || *c == ' ' || *c == '/')
			break;
		u->u_comm[i] = *c++;
	}
	u->u_comm[i] = '\0';
/*
 * Copy other process status values to the user area, as required.
 */
# endif	/* HPUX8 */

	return(0);
}
#endif	/* hpux */
